local MASQUE = LibStub("Masque", true);
local buttongroupActionButtons = nil;
local buttongroupBags = nil;

function CleanUI_InitActionBars()

    CleanUI_RegisterActionBarEvents();

    CleanUIUserData.scaleActionBars = CleanUIUserData.scaleActionBars or 1.0

    if (MASQUE) then
        buttongroupActionButtons = MASQUE:Group("CleanUI", "Action Buttons");
        buttongroupBags = MASQUE:Group("CleanUI", "Bags");
        
        CleanUI_SkinActionBars();
        CleanUI_SkinBagButtons();
    end

    hooksecurefunc("PetActionBar_Update", CleanUI_InitPetBarArtwork);
    hooksecurefunc(MainMenuBar, "SetPositionForStatusBars", CleanUI_SetPositionForStatusBars);

    hooksecurefunc("ActionButton_UpdateRangeIndicator", CleanUI_ActionButton_UpdateRangeIndicator);

    
    CleanUI_UpdateBagButtons();
    CleanUI_UpdateActionButtons();
    
    CleanUI_RemoveMacroNames();
    CleanUI_RemoveArtwork();
    CleanUI_InitActionBarArtwork();
    CleanUI_InitPetBarArtwork();
    CleanUI_ContainerFrame_UpdateFreeSlots();

    CleanUI_InitActionBarCombopoints();

    local frame = CreateFrame("Frame");
    frame:HookScript("OnUpdate", CleanUI_ActionBar_OnUpdate);
end

function CleanUI_RegisterActionBarEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ACTIONBAR_UPDATE_USABLE");

    frame:HookScript("OnEvent", CleanUI_ActionBarOnEvent);
end

local UPDATE_WAIT = 0.2;
local _WaitTimeSinceLast = 0;
function CleanUI_ActionBar_OnUpdate(self, elapsed)
    if (_WaitTimeSinceLast <= 0) then
        CleanUI_UpdateDeBuffs(elapsed);
        _WaitTimeSinceLast = UPDATE_WAIT;
    else
        _WaitTimeSinceLast = _WaitTimeSinceLast - elapsed;
    end
end

function CleanUI_ActionBarOnEvent(self, event, ...)

    if (event == "ACTIONBAR_UPDATE_USABLE") then
        CleanUI_UpdateActionButtons();
    end
end

function CleanUI_UpdateDeBuffs(elapsed)
    
    for i = 1, 12 do
        CleanUI_ActionButtonDeBuffs_OnUpdate(_G["ActionButton"..i], elapsed);
        CleanUI_ActionButtonDeBuffs_OnUpdate(_G["MultiBarBottomLeftButton"..i], elapsed);
        CleanUI_ActionButtonDeBuffs_OnUpdate(_G["MultiBarBottomRightButton"..i], elapsed);
        CleanUI_ActionButtonDeBuffs_OnUpdate(_G["MultiBarRightButton"..i], elapsed);
        CleanUI_ActionButtonDeBuffs_OnUpdate(_G["MultiBarLeftButton"..i], elapsed);
    end

    for i = 1, 12 do
        local button = _G["OverrideActionBarButton"..i];

        if (button) then
            CleanUI_ActionButtonDeBuffs_OnUpdate(button, elapsed);
        end
    end
end

function CleanUI_UpdateActionButtons()
    for i = 1, 12 do
        CleanUI_ActionButton_OnUpdate(_G["ActionButton"..i]);
        CleanUI_ActionButton_OnUpdate(_G["MultiBarBottomLeftButton"..i]);
        CleanUI_ActionButton_OnUpdate(_G["MultiBarBottomRightButton"..i]);
        CleanUI_ActionButton_OnUpdate(_G["MultiBarRightButton"..i]);
        CleanUI_ActionButton_OnUpdate(_G["MultiBarLeftButton"..i]);
    end

    for i = 1, 12 do
        local button = _G["OverrideActionBarButton"..i];

        if (button) then
            CleanUI_ActionButton_OnUpdate(button);
        end
    end
end

function CleanUI_InitActionBarCombopoints()
    local maxComboPoints = UnitPowerMax(PlayerFrame.unit, Enum.PowerType.ComboPoints);

    if (maxComboPoints == 0) then
        return;
    end

    MainMenuBar.combopoints = CreateFrame("Frame", "CUIMainMenuBarComboPoints", MainMenuBar);
    MainMenuBar.combopoints:ClearAllPoints();
    MainMenuBar.combopoints:SetSize(10, 10);
    MainMenuBar.combopoints:SetPoint("TOPLEFT", MainMenuBarArtFrame, "TOPLEFT", 0, -11);
    MainMenuBar.combopoints:SetPoint("BOTTOMRIGHT", MainMenuBarArtFrame, "TOPRIGHT", 0, -14);

    local glowSize = 100;

    MainMenuBar.combopointsGlow = CleanUIActionBarArtFrame:CreateTexture("CUIMainMenuBarComboPointsGlow", "BACKGROUND");
    MainMenuBar.combopointsGlow:SetPoint("TOPLEFT", CleanUIActionBarArtFrame, "TOPLEFT", -glowSize, glowSize/1.5);
    MainMenuBar.combopointsGlow:SetPoint("BOTTOMRIGHT", CleanUIActionBarArtFrame, "BOTTOMRIGHT", glowSize, 0);
    MainMenuBar.combopointsGlow:SetTexture("Interface\\AddOns\\CleanUI\\skins\\frame\\frame-glow");
    MainMenuBar.combopointsGlow:SetVertexColor(0.0, 1.0, 0.4);
    MainMenuBar.combopointsGlow:SetAlpha(0.0);

    local parent = MainMenuBar.combopoints;

	parent:RegisterEvent("PLAYER_TARGET_CHANGED");
	parent:RegisterEvent("UNIT_POWER_FREQUENT");
	parent:RegisterEvent("UNIT_MAXPOWER");
	parent:RegisterEvent("PLAYER_ENTERING_WORLD");
    
    parent:RegisterEvent("PLAYER_REGEN_DISABLED");
    parent:RegisterEvent("PLAYER_REGEN_ENABLED");
    
    parent:HookScript("OnEvent", CleanUI_ComboPointsOnEvent);

    local last = nil;

    for i=1, maxComboPoints do
        local combo = _G[parent:GetName()..i];
        if (combo == nil) then
            combo = parent:CreateTexture(parent:GetName()..i, "ARTWORK");
            combo:SetWidth((parent:GetWidth() + 2) / maxComboPoints - 3);
            combo:SetHeight(parent:GetHeight());
            combo:SetTexture("Interface\\TargetingFrame\\UI-StatusBar");

            combo:SetAlpha(0.0);
        end

        if (last) then
            combo:ClearAllPoints();
            combo:SetPoint("LEFT", last, "RIGHT", 3, 0);
        else
            combo:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, 0);
        end

        last = combo;
    end
end

local lastComboPoints = 0

function CleanUI_ComboPointsOnEvent(self, event, ...)
    local comboPoints = UnitPower(PlayerFrame.unit, Enum.PowerType.ComboPoints);    
    local maxComboPoints = UnitPowerMax(PlayerFrame.unit, Enum.PowerType.ComboPoints);

    if (lastComboPoints == comboPoints) then
        return;
    end

    lastComboPoints = comboPoints;

    local r = 0;
    local g = 1;
    local b = 0;

    for i=1, maxComboPoints do
        local combo = _G[self:GetName()..i];

        if (comboPoints < maxComboPoints) then
            r = 1.0;
            g = 1.0 / maxComboPoints * i;
            b = 0.2;
        end

        combo:SetVertexColor(r, g, b);

        if (i > comboPoints and combo:GetAlpha() >= 0) then
            UIFrameFadeOut(combo, 0.1, combo:GetAlpha(), 0.0);
        end


        if (i <= comboPoints and combo:GetAlpha() == 0) then
            UIFrameFadeIn(combo, 0.4, combo:GetAlpha(), 1.0);
        end
    end

    if (InCombatLockdown()) then
        if (CleanUISavedVariables.showComboPointsGlow) then
            if (comboPoints == maxComboPoints) then
                UIFrameFadeIn(MainMenuBar.combopointsGlow, 0.4, MainMenuBar.combopointsGlow:GetAlpha(), 0.35);
            else
                UIFrameFadeOut(MainMenuBar.combopointsGlow, 0.1, MainMenuBar.combopointsGlow:GetAlpha(), 0.0);
            end
        end
    else
        MainMenuBar.combopointsGlow:Hide();
    end
end

function CleanUI_SkinActionBars()
    for i = 1, 12 do
        CleanUI_SkinActionButton(_G["ActionButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarBottomLeftButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarBottomRightButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarRightButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarLeftButton"..i]);
        
        _G["ActionButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarBottomLeftButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarBottomRightButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarRightButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarLeftButton"..i]:SetFrameStrata("MEDIUM");
    end

    for i = 1, 10 do
        CleanUI_SkinActionButton(_G["StanceButton"..i]);
        CleanUI_SkinActionButton(_G["PetActionButton"..i]);
    end
end

function CleanUI_SkinActionButton(button)
    buttongroupActionButtons:AddButton(button);
end

function CleanUI_SkinBagButtons()
    for i=0, 3, 1 do
        buttongroupBags:AddButton(_G["CharacterBag"..i.."Slot"]);
        local iconTexture = _G["CharacterBag"..i.."SlotIconTexture"];
        iconTexture:SetDrawLayer("BACKGROUND");
        iconTexture:SetTexCoord(0.08, 0.92, 0.08, 0.92);
        
        _G["CharacterBag"..i.."Slot"].IconBorder:SetTexture(nil);
        _G["CharacterBag"..i.."Slot"].IconBorder:SetSize(0.1, 0.1);
    end
    
    buttongroupBags:AddButton(MainMenuBarBackpackButton);
    local iconTexture = _G["MainMenuBarBackpackButtonIconTexture"];
    iconTexture:SetDrawLayer("BACKGROUND");
    iconTexture:SetTexCoord(0.08, 0.92, 0.08, 0.92);
end

function CleanUI_RemoveMacroNames()
    for i = 1, 12 do
        _G["ActionButton"..i.."Name"]:Hide();
        _G["MultiBarBottomLeftButton"..i.."Name"]:Hide();
        _G["MultiBarBottomRightButton"..i.."Name"]:Hide();
        _G["MultiBarRightButton"..i.."Name"]:Hide();
        _G["MultiBarLeftButton"..i.."Name"]:Hide();
    end
end

function CleanUI_RemoveArtwork()
    MainMenuBarArtFrame.LeftEndCap:SetTexture();
    MainMenuBarArtFrame.RightEndCap:SetTexture();

    StatusTrackingBarManager.SingleBarLarge:SetTexture();
    StatusTrackingBarManager.SingleBarLargeUpper:SetTexture();
    StatusTrackingBarManager.SingleBarSmall:SetTexture();
    StatusTrackingBarManager.SingleBarSmallUpper:SetTexture();

    for i, bar in ipairs(StatusTrackingBarManager.bars) do
		bar.StatusBar.Background:SetTexture();
	end
end

function CleanUI_InitActionBarArtwork()
    local cuiArtFrame = _G["CleanUIActionBarArtFrame"];
    if (not cuiArtFrame) then
        cuiArtFrame = CreateFrame("Frame", "CleanUIActionBarArtFrame", MainMenuBarArtFrame);
        cuiArtFrame:SetFrameStrata("LOW");
    end

    CleanUI_SetPositionForStatusBars()
    
    local texture = "Interface\\PLAYERACTIONBARALT\\"..CleanUI_GetFrameStyle();
    
    if (not cuiArtFrame.background) then
        cuiArtFrame.background = cuiArtFrame:CreateTexture("ARTWORK");
        cuiArtFrame.background:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", -2, 0);
        cuiArtFrame.background:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMRIGHT", 0, 0);
        cuiArtFrame.background:SetTexture(texture);
        cuiArtFrame.background:SetTexCoord(0.0, 1.0, 0.1757, 0.3555);
    end
    
    if (not cuiArtFrame.topborder) then
        cuiArtFrame.topborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.topborder:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", 0, 0);
        cuiArtFrame.topborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "TOPRIGHT", 0, -3);
        cuiArtFrame.topborder:SetTexture(texture);
        cuiArtFrame.topborder:SetTexCoord(0.0, 1.0, 0.0898, 0.0977);
    end

    -- gryphons
    
    if (not cuiArtFrame.leftborder) then
        cuiArtFrame.leftborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.leftborder:SetTexture(texture);
        cuiArtFrame.leftborder:SetTexCoord(0, 0.32, 0.775, 0.996);
        cuiArtFrame.leftborder:SetPoint("TOPRIGHT", cuiArtFrame, "TOPLEFT", 30, 10);
        cuiArtFrame.leftborder:SetPoint("BOTTOMLEFT", cuiArtFrame, "BOTTOMLEFT", -143, 0);
    end
    
    if (not cuiArtFrame.rightborder) then
        cuiArtFrame.rightborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.rightborder:SetTexture(texture);
        cuiArtFrame.rightborder:SetTexCoord(0.32, 0, 0.775, 0.996);
        cuiArtFrame.rightborder:SetPoint("TOPLEFT", cuiArtFrame, "TOPRIGHT", -30, 10);
        cuiArtFrame.rightborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMRIGHT", 143, 0);
    end

    -- /run CleanUIActionBarArtFrame:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", -145, 45);
    -- /run CleanUIActionBarArtFrame.leftborder:SetTexCoord(0, 0.32, 0.72, 1);
end

function CleanUI_SetPositionForStatusBars()
    local cuiArtFrame = _G["CleanUIActionBarArtFrame"];
    if (cuiArtFrame) then
        local topleft = ActionButton1;
    
        if (MultiBarBottomLeft:IsShown()) then
            topleft = MultiBarBottomLeftButton1;
        end

        cuiArtFrame:SetParent(MainMenuBarArtFrame);
        cuiArtFrame:SetPoint("TOPLEFT", topleft, "TOPLEFT", -8, 5);

        local visibleBars = StatusTrackingBarManager:GetNumberVisibleBars();

        CleanUIActionBarArtFrame:SetPoint("BOTTOMRIGHT", MainMenuBarArtFrame, "BOTTOMRIGHT", 2, -27);

        if ( visibleBars == 2 ) then 
            cuiArtFrame:SetPoint("BOTTOMRIGHT", MainMenuBarArtFrame, "BOTTOMRIGHT", 2, -30);
        elseif ( visibleBars == 1 ) then
            cuiArtFrame:SetPoint("BOTTOMRIGHT", MainMenuBarArtFrame, "BOTTOMRIGHT", 2, -24);
        else 
            cuiArtFrame:SetPoint("BOTTOMRIGHT", MainMenuBarArtFrame, "BOTTOMRIGHT", 2, -10);
        end
    end
end

function CleanUI_InitPetBarArtwork()    
    -- pet bar 
    local firstPetButton = _G["PetActionButton1"];
    local lastPetButton = _G["PetActionButton1"];
    local button;

    for i = 2, 10 do
        button = _G["PetActionButton"..i];
        if (button and button:IsVisible()) then
            lastPetButton = button;
        end
    end
    
    local cuiPetArtFrame = _G["CleanUIPetBarArtFrame"];
    if (not cuiPetArtFrame) then
        cuiPetArtFrame = CreateFrame("Frame", "CleanUIPetBarArtFrame", PetActionBarFrame);
    end

    cuiPetArtFrame:SetParent(PetActionBarFrame);
    cuiPetArtFrame:SetPoint("TOPLEFT", firstPetButton, "TOPLEFT", -12, 5);
    cuiPetArtFrame:SetPoint("BOTTOMRIGHT", lastPetButton, "BOTTOMRIGHT", 12, -3);
    cuiPetArtFrame:SetFrameStrata("BACKGROUND");
    
    local texture = "Interface\\PLAYERACTIONBARALT\\"..CleanUI_GetFrameStyle();
    
    if (not cuiPetArtFrame.background) then
        cuiPetArtFrame.background = cuiPetArtFrame:CreateTexture("ARTWORK");
        cuiPetArtFrame.background:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPLEFT", 0, 0);
        cuiPetArtFrame.background:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "BOTTOMRIGHT", 0, 0);
        cuiPetArtFrame.background:SetTexture(texture);
        cuiPetArtFrame.background:SetTexCoord(0.0, 1.0, 0.1757, 0.3555);
    end    
    
    if (not cuiPetArtFrame.topborder) then
        cuiPetArtFrame.topborder = cuiPetArtFrame:CreateTexture("HIGHLIGHT");
        cuiPetArtFrame.topborder:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPLEFT", 0, 0);
        cuiPetArtFrame.topborder:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "TOPRIGHT", 0, -3);
        cuiPetArtFrame.topborder:SetTexture(texture);
        cuiPetArtFrame.topborder:SetTexCoord(0.0, 1.0, 0.0898, 0.0977);
    end
    
    if (not cuiPetArtFrame.leftborder) then
        cuiPetArtFrame.leftborder = cuiPetArtFrame:CreateTexture("HIGHLIGHT");
        cuiPetArtFrame.leftborder:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPLEFT", -2, 3);
        cuiPetArtFrame.leftborder:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "BOTTOMLEFT", 24, -3);
        cuiPetArtFrame.leftborder:SetTexture(texture);
        cuiPetArtFrame.leftborder:SetTexCoord(0.2402, 0.3027, 0.7910, 0.8750);
    end
    
    if (not cuiPetArtFrame.rightborder) then
        cuiPetArtFrame.rightborder = cuiPetArtFrame:CreateTexture("HIGHLIGHT");
        cuiPetArtFrame.rightborder:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPRIGHT", 2, 3);
        cuiPetArtFrame.rightborder:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "BOTTOMRIGHT", -24, -3);
        cuiPetArtFrame.rightborder:SetTexture(texture);
        cuiPetArtFrame.rightborder:SetTexCoord(0.3027, 0.2402, 0.7910, 0.8750);
    end
end

function CleanUI_UpdateBagButtons()
    MainMenuBarBackpackButton:SetParent(UIParent);

    CharacterBag0Slot:SetParent(UIParent);
    CharacterBag1Slot:SetParent(UIParent);
    CharacterBag2Slot:SetParent(UIParent);
    CharacterBag3Slot:SetParent(UIParent);

    MainMenuBarBackpackButton:ClearAllPoints();
    MainMenuBarBackpackButton:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -89, 2.2);

    MainMenuBarBackpackButton:SetScale(0.9);

    CharacterBag0Slot:SetScale(1.2);
    CharacterBag0SlotStock:SetScale(0.75);
    CharacterBag0Slot:ClearAllPoints();
    CharacterBag0Slot:SetPoint("RIGHT", MainMenuBarBackpackButton, "LEFT", -2.7, 0);

    CharacterBag1Slot:SetScale(1.2);
    CharacterBag1SlotStock:SetScale(0.75);
    CharacterBag1Slot:ClearAllPoints();
    CharacterBag1Slot:SetPoint("RIGHT", CharacterBag0Slot, "LEFT", -2.7, 0);

    CharacterBag2Slot:SetScale(1.2);
    CharacterBag2SlotStock:SetScale(0.75);
    CharacterBag2Slot:ClearAllPoints();
    CharacterBag2Slot:SetPoint("RIGHT", CharacterBag1Slot, "LEFT", -2.7, 0);

    CharacterBag3Slot:SetScale(1.2);
    CharacterBag3SlotStock:SetScale(0.75);
    CharacterBag3Slot:ClearAllPoints();
    CharacterBag3Slot:SetPoint("RIGHT", CharacterBag2Slot, "LEFT", -2.7, 0);
    
    CleanUI_InitBagButton(MainMenuBarBackpackButton);
    CleanUI_InitBagButton(CharacterBag0Slot);
    CleanUI_InitBagButton(CharacterBag1Slot);
    CleanUI_InitBagButton(CharacterBag2Slot);
    CleanUI_InitBagButton(CharacterBag3Slot);
end

function CleanUI_InitBagButton(button)
    button:HookScript("OnEnter", CleanUI_BagButton_OnEnter);
    button:HookScript("OnLeave", CleanUI_BagButton_OnLeave);
    CleanUI_BagButton_OnLeave(button)
end

function CleanUI_BagButton_OnEnter(self)
    self:SetAlpha(1.0);
end

function CleanUI_BagButton_OnLeave(self)
    self:SetAlpha(0.7);
end

function CleanUI_ActionButton_OnUpdate(self)

    local desaturated = nil;
    local r, g, b = 1.0, 1.0, 1.0;

    if (self.action) then
        local valid = IsActionInRange(self.action);
        
        if (valid ~= false) then
            self.icon:SetDesaturated(nil);
            r, g, b = 1.0, 1.0, 1.0;
            
            local isUsable, notEnoughMana = IsUsableAction(self.action);
            
            if (notEnoughMana) then
                local powerType, powerTypeString = UnitPowerType("player");
                local color = PowerBarColor[powerTypeString];
                
                desaturated = 1;
                r, g, b = color.r/2, color.g/2, color.b/2;
            elseif (not isUsable) then
                desaturated = 1;
                r, g, b = 0.0, 0.4, 0.2;
            end
        else
            desaturated = 1;
            r, g, b = 1.0, 0.0, 0.0;
        end
    end

    self.icon:SetDesaturated(desaturated);
    self.icon:SetVertexColor(r, g, b);
end

-- Shared between the action bar and the pet bar.
function CleanUI_ActionButton_UpdateRangeIndicator(self, checksRange, inRange)
    CleanUI_ActionButton_OnUpdate(self);
end

