function CleanUI_InitAuctions()
    CleanUI_RegisterAuctionsEvents();
end

function CleanUI_RegisterAuctionsEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");

    frame:HookScript("OnEvent", CleanUI_AuctionsOnEvent);
end

function CleanUI_AuctionsOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_AuctionHouseUI") then
            hooksecurefunc("BattlePetToolTip_Show", CleanUI_Pets_BattlePetToolTip_Show);
            hooksecurefunc(AuctionHouseUtil, "SetAuctionHouseTooltip", CleanUI_SetAuctionHouseTooltip);
        end
    end
end

local function getModelScene()
    if (not AuctionHouseFrame.cuiModelScene) then
        AuctionHouseFrame.cuiModelScene = CreateFrame("ModelScene", 'CUI_Auctions_ModelScene', AuctionHouseFrame, "ModelSceneMixinTemplate, NonInteractableModelSceneMixinTemplate");
        AuctionHouseFrame.cuiModelScene:SetFrameStrata("TOOLTIP");

        AuctionHouseFrame.cuiModelScene:SetSize(400, AuctionHouseFrame:GetHeight() - 50);
        AuctionHouseFrame.cuiModelScene:SetPoint("TOPLEFT", AuctionHouseFrame, "TOPRIGHT", 5, -12);

        CleanUI_SetArtBackdrop(AuctionHouseFrame.cuiModelScene, 0, 0, -30, 0);
    end

    return AuctionHouseFrame.cuiModelScene;
end

local modelShown = false;

local function showModel(modelSceneID, displayID, tag)
    local modelScene = getModelScene();
    modelScene:TransitionToModelSceneID(modelSceneID, CAMERA_TRANSITION_TYPE_IMMEDIATE, CAMERA_MODIFICATION_TYPE_DISCARD, true);

    local actor = modelScene:GetActorByTag(tag);
    if ( actor ) then
        actor:SetModelByCreatureDisplayID(displayID);
        actor:SetAnimationBlendOperation(LE_MODEL_BLEND_OPERATION_NONE);
    end

    modelScene:Show();
    modelShown = true;
end

local AuctionHouseTooltipType = {
	PetLink = 1;
	ItemLink = 2;
	ItemKey = 3;
};

local function getAuctionHouseTooltipType(rowData)
	if rowData.itemLink then
		local linkType = LinkUtil.ExtractLink(rowData.itemLink);
		if linkType == "battlepet" then
			return AuctionHouseTooltipType.PetLink, rowData.itemLink;
		elseif linkType == "item" then
			return AuctionHouseTooltipType.ItemLink, rowData.itemLink;
		end
	elseif rowData.itemKey then
		local restrictQualityToFilter = true;
		local itemKeyInfo = C_AuctionHouse.GetItemKeyInfo(rowData.itemKey, restrictQualityToFilter);
		if itemKeyInfo and itemKeyInfo.battlePetLink then
			return AuctionHouseTooltipType.PetLink, itemKeyInfo.battlePetLink;
		end

		return AuctionHouseTooltipType.ItemKey, rowData.itemKey;
	end

	return nil;
end

function CleanUI_SetAuctionHouseTooltip(owner, rowData)
    local modelScene = getModelScene();

    local tooltipType, data = getAuctionHouseTooltipType(rowData);
	if not tooltipType or tooltipType == AuctionHouseTooltipType.ItemKey then
        modelScene:Hide();
        modelShown = false;
		return;
	end
end

function CleanUI_Pets_BattlePetToolTip_Show(speciesID, level, breedQuality, maxHealth, power, speed, name)

    local petName = name;

    local start1 = string.find(name, "[", nil, true);
    local start2 = string.find(name, "]", nil, true);
    
    if (start1 and start1 >= 0 and start2 and start2 >= 0) then
        petName = string.sub(name, start1 +1, start2 - 1);
    end

    if (speciesID and speciesID > 0) then
        local petData = CleanUIPetData[petName];
        if (petData) then
            GameTooltip_SetDefaultAnchor(GameTooltip, BattlePetTooltip);

            GameTooltip:AddLine(petData.tooltip, 1, 1, 1, true);
            GameTooltip:AddLine(petData.description, nil, nil, nil, true);

            GameTooltip:Show();

            local _, _, _, _, _, _, _, _, _, _, _, displayID = C_PetJournal.GetPetInfoBySpeciesID(speciesID);
            local _, loadoutModelSceneID = C_PetJournal.GetPetModelSceneInfoBySpeciesID(speciesID);
            showModel(loadoutModelSceneID, displayID, "pet");
        end
    end
end

local function GetAuctionButton(buttonType, id)
	if ( buttonType == "owner" ) then
		return _G["AuctionsButton"..id];
	elseif ( buttonType == "bidder" ) then
		return _G["BidButton"..id];
	elseif ( buttonType == "list" ) then
		return _G["BrowseButton"..id];
	end
end
