local MASQUE = LibStub("Masque", true);
local buttongroupItems = nil;
    
CleanUIContainerLock = CleanUIContainerLock or {}
CleanUIContainerSpecialQuestItems = CleanUIContainerSpecialQuestItems or {};

-- temporary data
local item_profit = 0;

local MAX_ADDITIONAL_BUTTONS = 7;

function CleanUI_InitBags()

    CleanUI_BagsHookEvents();
    CleanUI_RegisterBagsEvents();

    if (MASQUE) then
        buttongroupItems = MASQUE:Group("CleanUI", "ItemBar");
        
        CleanUI_SkinBagButtons();
    end

    -- Display the amount of copper that is added to the tooltip
    GameTooltip:HookScript("OnTooltipAddMoney",
        function(self, amount)
            item_profit = amount;
        end);
    
    CleanUI_CreateItemButtons();

    MicroButtonAndBagsBar:Hide();
    
    BackpackTokenFrameToken1Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    BackpackTokenFrameToken2Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    BackpackTokenFrameToken3Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);

    -- update backback button positions
    CharacterBag0Slot:ClearAllPoints();
    CharacterBag0Slot:SetPoint("BOTTOMRIGHT", MainMenuBarBackpackButton, "BOTTOMLEFT", -2, 0);

    CleanUI_InitAdditionalItemButtons();
end

function CleanUI_RegisterBagsEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("MERCHANT_SHOW");
    
    frame:HookScript("OnEvent", CleanUI_BagsOnEvent);
end

function CleanUI_BagsOnEvent(self, event, ...)
    if (event == "MERCHANT_SHOW") then
        CleanUI_MerchantShow();
    end
end

function CleanUI_BagsHookEvents()
    hooksecurefunc("ContainerFrame_Update", CleanUI_ContainerFrame_Update);
    hooksecurefunc("ContainerFrameItemButton_OnModifiedClick", CleanUI_ContainerFrameItemButton_OnModifiedClick);

    hooksecurefunc("ContainerFrame_GenerateFrame", CleanUI_ContainerFrame_GenerateFrame);
    hooksecurefunc("ContainerFrameItemButton_SetForceExtended", CleanUI_ContainerFrameItemButton_SetForceExtended);

    -- Tooltip für MoneyFrame   
    ContainerFrame1MoneyFrameGoldButton:HookScript("OnEnter", CleanUI_ShowMoneyFrameTooltip);
    ContainerFrame1MoneyFrameGoldButton:HookScript("OnLeave", CleanUI_HideMoneyFrameTooltip);      
    ContainerFrame1MoneyFrameSilverButton:HookScript("OnEnter", CleanUI_ShowMoneyFrameTooltip);
    ContainerFrame1MoneyFrameSilverButton:HookScript("OnLeave", CleanUI_HideMoneyFrameTooltip);  
    ContainerFrame1MoneyFrameCopperButton:HookScript("OnEnter", CleanUI_ShowMoneyFrameTooltip);
    ContainerFrame1MoneyFrameCopperButton:HookScript("OnLeave", CleanUI_HideMoneyFrameTooltip);
end

function CleanUI_ContainerFrameItemButton_OnModifiedClick(self, button)

    if (IsAltKeyDown()) then
        if (button == "LeftButton") then

            -- item lock

            local itemLink = GetContainerItemLink(self:GetParent():GetID(), self:GetID());

            if (itemLink) then
                -- lock state can't be removed by modified click
                if (not CleanUI_IsLockedItem(itemLink)) then
                    CleanUI_AddLockedItem(itemLink);
                end
            end

            CleanUI_UpdateAllContainers();
        elseif (button == "RightButton") then

            -- ignore quest item

            local isQuestItem, questId, isActive = GetContainerItemQuestInfo(self:GetParent():GetID(), self:GetID());

            if (isQuestItem) then
                local itemId = GetContainerItemID(self:GetParent():GetID(), self:GetID());

                if (CleanUI_IsHiddenSpecialQuestItem(itemId)) then
                    CleanUI_ShowSpecialQuestItem(itemId)
                else
                    CleanUI_HideSpecialQuestItem(itemId)
                end

                CleanUI_UpdateAllContainers();
            end
        end
    end
end

function CleanUI_UpdateAllContainers()
    for i=1, NUM_CONTAINER_FRAMES, 1 do
        local frame = _G["ContainerFrame"..i];
        if (frame and frame:IsShown()) then
            ContainerFrame_Update(frame);
        end
    end
end

function CleanUI_IsLockedItem(link)
    local itemName = GetItemInfo(link);
    local foundItemName;

    for index, foundLink in ipairs(CleanUIContainerLock) do
        foundItemName = GetItemInfo(foundLink);

        if (foundItemName == itemName) then
            return true;
        end
    end

    return false;
end

function CleanUI_AddLockedItem(link)
    tinsert(CleanUIContainerLock, link);
end

function CleanUI_RemoveLockedItem(link)
    local itemName = GetItemInfo(link);
    local foundItemName;

    for index, foundLink in ipairs(CleanUIContainerLock) do
        foundItemName = GetItemInfo(foundLink);

        if (foundItemName == itemName) then
            tremove(CleanUIContainerLock, index);

            CleanUI_UpdateAllContainers();
            return true;
        end
    end
end

function CleanUI_IsHiddenSpecialQuestItem(itemId)
    return tContains(CleanUIContainerSpecialQuestItems, itemId);
end

function CleanUI_HideSpecialQuestItem(itemId)
    if (not CleanUI_IsHiddenSpecialQuestItem(itemId)) then
        tinsert(CleanUIContainerSpecialQuestItems, itemId);
    end
end

function CleanUI_ShowSpecialQuestItem(itemId)
    local hiddenItems = {};

    for index, foundId in ipairs(CleanUIContainerSpecialQuestItems) do
        if (itemId ~= foundId and not tContains(hiddenItems, foundId)) then
            tinsert(hiddenItems, foundId);
        end
    end

    CleanUIContainerSpecialQuestItems = hiddenItems;

    CleanUI_UpdateAllContainers();
end

function CleanUI_MerchantShow()
    -- auto repair
    if (not IsControlKeyDown() and CanMerchantRepair()) then
        repairAllCost, canRepair = GetRepairAllCost();

        if (IsAltKeyDown()) then
            RepairAllItems(1);      -- guild
        else
            RepairAllItems();
        end
        
        PlaySound(SOUNDKIT.ITEM_REPAIR);

        if (repairAllCost > 0) then
            DEFAULT_CHAT_FRAME:AddMessage(BAGS_REPAIR_DONE.." ("..GetCoinTextureString(repairAllCost)..")", 1.0, 1.0, 0, 1, 10);
        end
    end

    local sum_item_profit = 0

    local container, containerid, numberOfSlots;
    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture;
    for id = 0, 4, 1 do
        containerid = "ContainerFrame"..(id + 1);
        container = getglobal(containerid);

        if (container) then
            numberOfSlots = GetContainerNumSlots(id);

            for slot=1, numberOfSlots, 1 do
                itemLink = GetContainerItemLink(id, numberOfSlots - slot + 1);

                if (itemLink) then
                    itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);

                    if (itemRarity == 0) then
                        GameTooltip:SetBagItem(id, numberOfSlots - slot + 1);

                        if (item_profit and item_profit > 0) then
                            sum_item_profit = sum_item_profit + item_profit;
                            DEFAULT_CHAT_FRAME:AddMessage(itemLink.. " "..BAGS_WAS_SOLD.." ("..GetCoinTextureString(item_profit)..")", 1.0, 1.0, 0, 1, 10);
                            UseContainerItem(id, numberOfSlots - slot + 1);
                        end
                    end
                end
            end
        end
    end

    --if (sum_item_profit > 0) then
    --    DEFAULT_CHAT_FRAME:AddMessage(BAGS_TOTAL..": "..GetCoinTextureString(sum_item_profit), 1.0, 1.0, 0, 1, 10);
    --end
end

function CleanUI_ContainerFrame_Update(frame)
    local id = frame:GetID();
    local name = frame:GetName();

    local itemButton, itemLink, itemId;

    for i=1, frame.size, 1 do
        itemButton = _G[name.."Item"..i];
        itemLink = GetContainerItemLink(id, itemButton:GetID());
        itemId = GetContainerItemID(id, itemButton:GetID());

        CleanUI_UpdateSpecialQuestItemState(itemButton, itemId);
        CleanUI_UpdateItemLock(itemButton, itemLink);
        CleanUI_ContainerFrame_UpdateItemLevel(itemButton, itemLink);
    end

    CleanUI_ContainerFrame_UpdateFreeSlots()
end

function CleanUI_ContainerFrame_UpdateFreeSlots()
    -- called from ActionBars
    SetItemButtonStock(MainMenuBarBackpackButton, GetContainerNumFreeSlots(0));
    SetItemButtonStock(CharacterBag0Slot, GetContainerNumFreeSlots(1));
    SetItemButtonStock(CharacterBag1Slot, GetContainerNumFreeSlots(2));
    SetItemButtonStock(CharacterBag2Slot, GetContainerNumFreeSlots(3));
    SetItemButtonStock(CharacterBag3Slot, GetContainerNumFreeSlots(4));
end

function CleanUI_UpdateSpecialQuestItemState(itemButton, itemId)
    local hidden = CleanUI_IsHiddenSpecialQuestItem(itemId)

    if (not itemButton.specialQuestItemState) then
        itemButton.specialQuestItemState = itemButton:CreateTexture("ARTWORK");
        itemButton.specialQuestItemState:SetPoint("TOPLEFT", itemButton, "TOPLEFT", 2, -2);
        itemButton.specialQuestItemState:SetPoint("BOTTOMRIGHT", itemButton, "TOPLEFT", 16, -16);
        itemButton.specialQuestItemState:SetTexture("Interface\\Common\\Indicator-Red");
    end

    if (hidden) then
        itemButton.specialQuestItemState:Show();
    else
        itemButton.specialQuestItemState:Hide();
    end
end

function CleanUI_UpdateItemLock(itemButton, itemLink)
    if (not itemLink) then
        CleanUI_UnLockItem(itemButton)
        return;
    end

    if (CleanUI_IsLockedItem(itemLink)) then
        CleanUI_LockItem(itemButton);
    else
        CleanUI_UnLockItem(itemButton);
    end

    --[[if (IsEquippableItem(itemLink)) then
        local itemIDs, foundLink, found;
        for index = 1, C_EquipmentSet.GetNumEquipmentSets() do
            local eqname, icon = C_EquipmentSet.GetEquipmentSetInfo(index);
            found = false;

            itemIDs = GetEquipmentSetItemIDs(eqname);

            for _, itemID in ipairs(itemIDs) do
                if (itemID > 0) then
                    _, foundLink = GetItemInfo(itemID);

                    local itemName = GetItemInfo(itemLink);
                    local foundName = GetItemInfo(foundLink);
                    
                    if (itemName == foundName) then
                        found = true;
                        break;
                    end
                end
            end
        end

        CleanUI_UpdateEquipmentSetMarker(itemButton, found);
    end]]
end

function CleanUI_UpdateEquipmentSetMarker(itemButton, found)
    if (not itemButton.eqIndicator) then
        itemButton.eqIndicator = itemButton:CreateFontString(itemButton:GetName().."EQIndicator", "ARTWORK");
        itemButton.eqIndicator:SetFontObject("NumberFontNormalSmallGray");
        itemButton.eqIndicator:SetSize(16, 16);
        itemButton.eqIndicator:ClearAllPoints();
        itemButton.eqIndicator:SetPoint("TOPLEFT", -1, 3);
        itemButton.eqIndicator:SetJustifyH("CENTER");
        itemButton.eqIndicator:SetText(NORMAL_FONT_COLOR_CODE.."●"..FONT_COLOR_CODE_CLOSE);
    end
    
    if (found) then
        itemButton.eqIndicator:Show();
    else
        itemButton.eqIndicator:Hide();
    end
end

function CleanUI_ContainerFrame_UpdateItemLevel(self, itemLink)
    -- create label
    local itemLevelLabel = _G[self:GetName().."ItemLevel"];
    if (not itemLevelLabel) then
        itemLevelLabel = self:CreateFontString(self:GetName().."ItemLevel", "ARTWORK");
        itemLevelLabel:SetFontObject("GameFontNormalSmall");
        itemLevelLabel:SetSize(self:GetWidth(), 10);
        itemLevelLabel:ClearAllPoints();
        itemLevelLabel:SetPoint("TOPRIGHT", -2, -2);
        itemLevelLabel:SetJustifyH("RIGHT");
        itemLevelLabel:SetText("");
    end
    
    -- no item link, no label
    if (itemLink == nil) then
        itemLevelLabel:Hide();
        return;
    end
    
    local _, _, quality, _, _, _, _, _, equipSlot = GetItemInfo(itemLink);
    
    -- no item link, no label
    if (not equipSlot or string.find(equipSlot, "INVTYPE") ~= 1) then
        itemLevelLabel:Hide();
        return;
    end
    
    -- show
    itemLevelLabel:Show();
    
    -- calculate level
    local effectiveILvl, isPreview, baseILvl = GetDetailedItemLevelInfo(itemLink);
    
    -- colorize
    local color = ITEM_QUALITY_COLORS[quality];
    local hexcolor = cui_rgb2hex(color.r, color.g, color.b);
    
    -- set text
    itemLevelLabel:SetText("|cff"..hexcolor..effectiveILvl.."|r");
end

function CleanUI_LockItem(itemButton)
    if (not itemButton:GetParent() or string.find(itemButton:GetParent():GetName(), "ContainerFrame") == nil) then
        return;
    end
    
    itemButton:RegisterForClicks();
    itemButton:RegisterForDrag();

    if (not itemButton.unlockButton) then
        itemButton.unlockButton = CreateFrame("Button", nil, itemButton, "UIPanelButtonTemplate");
        CleanUI_RemoveUiButtonTexture(itemButton.unlockButton);
        itemButton.unlockButton:SetSize(16, 16);
        itemButton.unlockButton:SetNormalTexture("Interface\\PETBATTLES\\PetBattle-LockIcon");
        itemButton.unlockButton:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
        itemButton.unlockButton:SetPoint("TOPLEFT", itemButton, "TOPLEFT", -1, 1);

        itemButton.unlockButton:SetScript("OnClick", CleanUI_UnLockClicked);
    end

    itemButton.unlockButton:Show();
end

function CleanUI_UnLockClicked(unlockButton)
    if (IsAltKeyDown()) then
        local itemLink = GetContainerItemLink(unlockButton:GetParent():GetParent():GetID(), unlockButton:GetParent():GetID());
        CleanUI_RemoveLockedItem(itemLink);
    end
end

function CleanUI_UnLockItem(itemButton)
    if (not itemButton:GetParent() or string.find(itemButton:GetParent():GetName(), "ContainerFrame") == nil) then
        return;
    end
    
    itemButton:RegisterForClicks("LeftButtonUp", "RightButtonUp");
    itemButton:RegisterForDrag("LeftButton");

    if (itemButton.unlockButton) then
        itemButton.unlockButton:Hide();
    end
end

function CleanUI_ContainerFrame_GenerateFrame(frame, size, id)
    frame:SetFrameStrata("HIGH");

    local name = frame:GetName();
    
	local bgTextureTop = _G[name.."BackgroundTop"];
	local bgTextureMiddle = _G[name.."BackgroundMiddle1"];
	local bgTextureMiddle2 = _G[name.."BackgroundMiddle2"];
	local bgTextureBottom = _G[name.."BackgroundBottom"];
    local bgTexture1Slot = _G[name.."Background1Slot"];
    
    bgTextureTop:SetDesaturated(1);
    bgTextureMiddle:SetDesaturated(1);
    bgTextureMiddle2:SetDesaturated(1);
    bgTextureBottom:SetDesaturated(1);
    bgTexture1Slot:SetDesaturated(1);

    bgTextureTop:SetVertexColor(0.5, 0.5, 0.5);
    bgTextureMiddle:SetVertexColor(0.5, 0.5, 0.5);
    bgTextureMiddle2:SetVertexColor(0.5, 0.5, 0.5);
    bgTextureBottom:SetVertexColor(0.5, 0.5, 0.5);
    bgTexture1Slot:SetVertexColor(0.5, 0.5, 0.5);

    local item;
    for slot = 1, GetContainerNumSlots(id) do
        itemButton = _G["ContainerFrame"..id.."Item"..slot];
        if (itemButton) then
            itemButton:GetNormalTexture():SetDesaturated(1);
            itemButton:GetNormalTexture():SetVertexColor(0.5, 0.5, 0.5);
        end
    end

    -- fix close button position
    local closeButton = _G[name.."CloseButton"];
    closeButton:ClearAllPoints();
    closeButton:SetPoint("TOPRIGHT", 0, 0);

    if (id == 1) then
        --UI-Backpack-TokenFrame
        local texture = CleanUI_FindTextureWith("UI-Backpack-TokenFrame", BackpackTokenFrame);

        if (texture) then
            texture:SetDesaturated(1);
            texture:SetVertexColor(0.5, 0.5, 0.5);
        end
    end
end

function CleanUI_ContainerFrameItemButton_SetForceExtended(itemButton, extended)
    if (extended) then
        itemButton:GetNormalTexture():SetVertexColor(0.4, 0.45, 0.5);
    else
        itemButton:GetNormalTexture():SetVertexColor(0.5, 0.5, 0.5);
    end
end

function CleanUI_ShowMoneyFrameTooltip(self)
    local playerFaction, localizedFaction = UnitFactionGroup("player");
    
    GameTooltip_SetDefaultAnchor(GameTooltip, self);
    
    GameTooltip:SetText(MONEY..":", 1.0, 1.0, 1.0);
    GameTooltip:AddLine(" ");
    
    local data = CleanUICurrency["money"] or {};
    
    local summe = 0;
    local name, moneyString;
    
    for guid, amount in pairs(data) do
        if (playerFaction == CleanUIAccountData[guid].faction) then
            summe = summe + amount;
            name = "|c333399ff"..CleanUIAccountData[guid].name.."|r";
            moneyString = "|cffffffff"..GetCoinTextureString(amount).."|r";
            GameTooltip:AddDoubleLine(name, moneyString);
        end
    end
    
    GameTooltip:AddLine(" ");
    GameTooltip:AddDoubleLine("|c333399ff"..localizedFaction.."|r", "|cffffffff"..GetCoinTextureString(summe).."|r");

    GameTooltip:Show();
end

function CleanUI_HideMoneyFrameTooltip()
    GameTooltip:FadeOut();
end

local function fillTable( tbl, ... )
    for i = 1, select("#", ...), 1 do
        local key = select(i, ...);
        key = tonumber(key);
        tinsert(tbl, key);
    end
end

local function indexOf( tbl, value )
    for index, found in ipairs(tbl) do
        if (found == value) then
            return index;
        end
    end
    
    return -1;
end

local function tableToString( tbl )
    local result = "";
    
    if (#tbl == 0) then
        return "0";
    end
    
    local lastWasItem = false;
    
    while (tbl[1] == 0) do
        tremove(tbl, 1);
    end
    
    while (tbl[#tbl] == 0) do
        tremove(tbl, #tbl);
    end
    
    for index, found in ipairs(tbl) do
        if (lastWasItem or found ~= 0) then
        
            if (index > 1) then
               result = result..",";
            end
            result = result..found;
        
            lastWasItem = (found ~= 0);
        end
    end
    
    return result;
end

local function saveAsItemButtons( tbl )
    CleanUIUserData.ItemButtonIDs = tableToString(tbl);
    CleanUI_CreateItemButtons();
end

function CleanUI_CreateItemButtons()
    if (CleanUIUserData.hideItemButtons) then
        return;
    end
    
    -- hide all existing buttons
    local index = 1;
    while ( _G["CUI_ItemButton"..index] ~= nil ) do
        _G["CUI_ItemButton"..index]:Hide();
        index = index + 1;
    end
    
    CleanUIUserData.ItemButtonIDs = CleanUIUserData.ItemButtonIDs or "142542";
        
    local ids = {};
    fillTable(ids, strsplit(",", CleanUIUserData.ItemButtonIDs));
    
    -- create new buttons
    local offset = 0;
    local index = 1;

    for i, id in ipairs(ids) do
        if (id == 0) then
            offset = offset + 8;
        else
            CleanUI_CreateItemButton("CUI_ItemButton"..index, id, offset, i);
            offset = offset + 40;
            index = index + 1;
        end
    end
end

function CleanUI_BagUpdateCooldown(button)
    local cooldown = button.cooldown;
    local itemId = button:GetAttribute("itemId");

    if (itemId ~= nil) then
        local start, duration, enable = GetItemCooldown(button:GetAttribute("itemId"));
        CooldownFrame_Set(cooldown, start, duration, enable);
    end
end

function CleanUI_CreateItemButton(name, id, offset, index)
    local button = _G[name];
    
    if (not button) then
        button = CreateFrame("Button", name, UIParent, "SecureActionButtonTemplate, ActionButtonTemplate");

        button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
        button:RegisterEvent("ACTIONBAR_UPDATE_STATE");
        button:RegisterEvent("BAG_UPDATE");
        button:RegisterEvent("ZONE_CHANGED");
        button:RegisterEvent("ZONE_CHANGED_INDOORS");
        button:RegisterEvent("ZONE_CHANGED_NEW_AREA");
        button:RegisterEvent("PLAYER_REGEN_DISABLED");
        button:RegisterEvent("PLAYER_REGEN_ENABLED");

        button:RegisterForDrag();
    end
    
    button:Show();
    
    -- Texture
    local itemTexture = GetItemIcon(id); 
    button.icon:SetTexture(itemTexture);
    
    button:SetPoint("TOPRIGHT", -3, -265 - offset);
    button:SetFrameStrata("LOW");
    
    if (MASQUE and buttongroupItems) then
        buttongroupItems:AddButton(button);
    end

    -- Item data

    button.asyncItem = Item:CreateFromItemID(id);

    button.asyncItem:ContinueOnItemLoad(function()
        local itemName, itemLink = GetItemInfo(id);
    
        button:SetAttribute("type1", "item");
        button:SetAttribute("type2", "item");
        button:SetAttribute("item", itemName);
        button:SetAttribute("itemId", id);
        button:SetAttribute("itemLink", itemLink);
        button:SetAttribute("index", index);
    
        CleanUI_CheckBagItemUsable(button);
    end);

    button.cooldown = _G[button:GetName().."Cooldown"];
    button.cooldown:ClearAllPoints();
    button.cooldown:SetPoint("TOPLEFT", button, "TOPLEFT", 0, -0);
    button.cooldown:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -0, 0);
    
    button.cooldown:SetEdgeTexture("Interface\\Cooldown\\edge");
    button.cooldown:SetSwipeColor(0, 0, 0);
    button.cooldown:SetHideCountdownNumbers(false);
    button.cooldown.currentCooldownType = COOLDOWN_TYPE_NORMAL;
    
    button:HookScript("OnEnter", CleanUI_ShowBagItemTooltip);
    button:HookScript("OnLeave", CleanUI_HideBagItemTooltip);
    button:HookScript("OnReceiveDrag", CleanUI_ItemButton_OnReceiveDrag);
    
    button:HookScript("OnEvent", CleanUI_ItemButton_OnEvent); 
    CleanUI_ItemButton_OnEvent(button, "BAG_UPDATE");
    
    button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
    button:HookScript("OnClick", CleanUI_ItemButton_OnClick);  
end

function CleanUI_ItemButton_OnReceiveDrag(self)
    local infoType, info1, info2 = GetCursorInfo();

    if (infoType == "item" and info1 ~= nil) then
        local index = self:GetAttribute("index");
        
        local ids = {};
        fillTable(ids, strsplit(",", CleanUIUserData.ItemButtonIDs));
        
        ids[index] = info1;
        saveAsItemButtons(ids);
    end

    ClearCursor();
end

function CleanUI_ItemButton_OnClick(self, button)
    if (button ~= "RightButton" or not IsControlKeyDown()) then
        return;
    end
    
    CleanUI_OpenItemButtonConfigMenu(self);
end

local function initItemButtonConfigMenu(self, menu)
    UIMenu_Initialize(menu);

    local countProfiles = 0;
    
    local ids = {};
    fillTable(ids, strsplit(",", CleanUIUserData.ItemButtonIDs));
    
    local itemId = self:GetAttribute("itemId");
    local index = indexOf(ids, itemId);
    
    if (index > 1) then
        UIMenu_AddButton(menu, ITEMBUTTON_MOVE_UP, nil, function(frame)
            tremove(ids, index);
            tinsert(ids, index - 1, itemId);
            saveAsItemButtons(ids);
        end);
    end
    
    if (index < #ids) then
        UIMenu_AddButton(menu, ITEMBUTTON_MOVE_DOWN, nil, function(frame)
            tremove(ids, index);
            tinsert(ids, index + 1, itemId);
            saveAsItemButtons(ids);
        end);
        
        local nextId = ids[index + 1];
    
        if (nextId ~= 0) then
            UIMenu_AddButton(menu, ITEMBUTTON_ADD_SEPARATOR, nil, function(frame)
                tinsert(ids, index + 1, 0);
                saveAsItemButtons(ids);
            end);
        end
    
        if (nextId == 0) then
            UIMenu_AddButton(menu, ITEMBUTTON_REMOVE_SEPARATOR, nil, function(frame)
                tremove(ids, index + 1);
                saveAsItemButtons(ids);
            end);
        end
    end
    
    UIMenu_AddButton(menu, ITEMBUTTON_ADD_NEW, nil, function(frame)
        tinsert(ids, index + 1, itemId);
        saveAsItemButtons(ids);
    end);
    
    UIMenu_AddButton(menu, ITEMBUTTON_REMOVE, nil, function(frame)
        tremove(ids, index);
        saveAsItemButtons(ids);
    end);
    
    UIMenu_AddButton(menu, "");
    UIMenu_AddButton(menu, CANCEL, nil, function(frame) self:Hide(); end);
    UIMenu_AutoSize(menu);
end

function CleanUI_OpenItemButtonConfigMenu(self)
    local menu = _G["CUI_ItemButtonConfigMenu"];
    if (not menu) then
        menu = CreateFrame("Frame", "CUI_ItemButtonConfigMenu", UIParent, "UIMenuTemplate");
    end
    initItemButtonConfigMenu(self, menu);
    menu:ClearAllPoints();
    menu:SetPoint("TOPRIGHT", self, "TOPLEFT", 0, 0);
    menu:Show();
end

function CleanUI_ItemButton_OnEvent(self, event, ...)
    if ( event == "ACTIONBAR_UPDATE_COOLDOWN" ) then
        CleanUI_BagUpdateCooldown(self);
    end
    if ( event == "BAG_UPDATE" or event == "ACTIONBAR_UPDATE_STATE" or event == "ZONE_CHANGED" or event == "ZONE_CHANGED_INDOORS"  or event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_REGEN_DISABLED" or event == "PLAYER_REGEN_ENABLED" ) then
        CleanUI_CheckBagItemUsable(self);
    end
    
    if ( event == "BAG_UPDATE") then
        local itemCount = GetItemCount(self:GetAttribute("itemId"));
        local counttext = _G[self:GetName().."Count"];
        
        if (itemCount > 1) then
            counttext:SetText(""..itemCount);
        else
            counttext:SetText("");
        end
    end
    
    if ( event == "CURSOR_UPDATE") then
        CleanUI_AdditionalItemButton_UpdateAlpha(self);
    end
end

function CleanUI_ShowBagItemTooltip(self)
    GameTooltip_SetDefaultAnchor(GameTooltip, self);

    local id = self:GetAttribute("itemId");

    if (id) then
        GameTooltip:SetItemByID(id);

        if (additionalInfo) then
            GameTooltip:AddLine(additionalInfo);
        end

        GameTooltip:Show();
    end
end

function CleanUI_ShowQuestItemTooltip(self)
    GameTooltip_SetDefaultAnchor(GameTooltip, self);
    GameTooltip:SetItemByID(self:GetAttribute("itemId"));

    GameTooltip:AddLine(BAGS_SPECIAL_QUESTITEM_INFO);

    GameTooltip:Show();
end

local function findItemByName(search)
    if (not search) then
        return false;
    end
    
    for bag = 0,4 do
        for slot = 1,GetContainerNumSlots(bag) do
            local item = GetContainerItemLink(bag,slot)
            if item and item:find(search) then
                return true;
            end
        end
    end
    
    return false;
end

function CleanUI_CheckBagItemUsable(self)
    
    if (self:GetAttribute("itemId") and not InCombatLockdown()) then
        local isUsable = IsUsableItem(self:GetAttribute("itemId"))
        local hasToy = PlayerHasToy(self:GetAttribute("itemId"))
        
        self:Show();
        if (isUsable or hasToy) then
            self.icon:SetAlpha(1.0);
        else
            self.icon:SetAlpha(0.3);
        end
    end
end

function CleanUI_HideBagItemTooltip()
    GameTooltip:FadeOut();
end

local function checkItemTooltip(link)
    local tooltip = _G["CUI_QuestItemCheckTooltip"];
    if (not tooltip) then
        tooltip = CreateFrame("GameTooltip", "CUI_QuestItemCheckTooltip", nil, "GameTooltipTemplate");
        tooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
    end

	tooltip:ClearLines();
    tooltip:SetHyperlink(link);
    
    local numLines = tooltip:NumLines();

    for i = 1, numLines do
        local text = _G["CUI_QuestItemCheckTooltipTextLeft"..i]:GetText() or "";
        
        if (text:find("^"..ITEM_SPELL_TRIGGER_ONUSE)) then
            return true;
        end
    end
    
    return false;
end

-- ADDITIONAL ITEM BUTTONS

local function saveAdditionalItemButtonIDs()
    local ids = "";
    for i = 1, MAX_ADDITIONAL_BUTTONS do
        local itemId = _G["CUI_MY_BUTTON"..i]:GetAttribute("itemId");
        ids = strjoin(",", ids, itemId or 0);
    end
    CleanUIUserData.additionalItemButtonIDs = ids;
end

local function updateButtonAttributes(self, info1)
    self.asyncItem = Item:CreateFromItemID(info1);

    self.asyncItem:ContinueOnItemLoad(function()
        local itemName, itemLink = GetItemInfo(info1);
    
        self:SetAttribute("type1", "item");
        self:SetAttribute("type2", "item");
        self:SetAttribute("item", itemName);
        self:SetAttribute("itemId", info1);
        self:SetAttribute("itemLink", itemLink);
    
        CleanUI_ItemButton_OnEvent(self, "BAG_UPDATE");
    
        -- Texture
        local itemTexture = GetItemIcon(info1); 
        self.icon:SetTexture(itemTexture);

        CleanUI_AdditionalItemButton_UpdateAlpha(self);

        saveAdditionalItemButtonIDs();
    end);
end

function CleanUI_InitAdditionalItemButtons()

    local button;

    -- create buttons on top of the bag buttons
    
    for i = 1, MAX_ADDITIONAL_BUTTONS do
        button = CleanUI_CreateAdditionalItemButton(i);
        button:SetPoint("BOTTOMRIGHT", -(2.5 + (i - 1) * 39), 43);
    end

    CleanUIUserData.additionalItemButtonIDs = CleanUIUserData.additionalItemButtonIDs or "0,0,0,0,0,0,0";

    local ids = {};
    fillTable(ids, strsplit(",", CleanUIUserData.additionalItemButtonIDs));
    
    for i = 1, MAX_ADDITIONAL_BUTTONS do
        if (ids[i] ~= 0) then
            updateButtonAttributes(_G["CUI_MY_BUTTON"..i], ids[i]);
        end
    end
    
end

function CleanUI_CreateAdditionalItemButton(index)

    local name = "CUI_MY_BUTTON"..index;
    local button = _G[name];
    
    if (not button) then
        button = CreateFrame("Button", name, UIParent, "SecureActionButtonTemplate, ActionButtonTemplate");
        button:SetFrameStrata("LOW");

        button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
        button:RegisterEvent("ACTIONBAR_UPDATE_STATE");
        button:RegisterEvent("BAG_UPDATE");
        button:RegisterEvent("ZONE_CHANGED");
        button:RegisterEvent("ZONE_CHANGED_INDOORS");
        button:RegisterEvent("ZONE_CHANGED_NEW_AREA");
        button:RegisterEvent("PLAYER_REGEN_DISABLED");
        button:RegisterEvent("PLAYER_REGEN_ENABLED");
        button:RegisterEvent("CURSOR_UPDATE");

        button:RegisterForDrag("LeftButton", "RightButton");
    
        if (MASQUE and buttongroupItems) then
            buttongroupItems:AddButton(button);
        end

        CleanUI_AdditionalItemButton_UpdateAlpha(button);

        button.cooldown = _G[button:GetName().."Cooldown"];
        button.cooldown:ClearAllPoints();
        button.cooldown:SetPoint("TOPLEFT", button, "TOPLEFT", 0, -0);
        button.cooldown:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -0, 0);
        
        button.cooldown:SetEdgeTexture("Interface\\Cooldown\\edge");
        button.cooldown:SetSwipeColor(0, 0, 0);
        button.cooldown:SetHideCountdownNumbers(false);
        button.cooldown.currentCooldownType = COOLDOWN_TYPE_NORMAL;
        
        button:HookScript("OnEnter", CleanUI_ShowBagItemTooltip);
        button:HookScript("OnLeave", CleanUI_HideBagItemTooltip);
        button:HookScript("OnDragStart", CleanUI_AdditionalItemButton_OnDragStart);
        button:HookScript("OnReceiveDrag", CleanUI_AdditionalItemButton_OnReceiveDrag);
        
        button:HookScript("OnEvent", CleanUI_ItemButton_OnEvent); 
        CleanUI_ItemButton_OnEvent(button, "BAG_UPDATE");
        
        button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
        button:HookScript("OnClick", CleanUI_ItemButton_OnClick);
    end

    button:Show();

    return button;
end

function CleanUI_AdditionalItemButton_OnDragStart(self, button)
    if ( LOCK_ACTIONBAR ~= "1" or IsModifiedClick("PICKUPACTION") ) then     
        PickupItem(self:GetAttribute("itemId"));
        
        self:SetAttribute("type1", nil);
        self:SetAttribute("type2", nil);
        self:SetAttribute("item", nil);
        self:SetAttribute("itemId", nil);
        self:SetAttribute("itemLink", nil);

        self.icon:SetTexture(nil);
        _G[self:GetName().."Count"]:SetText("");

        saveAdditionalItemButtonIDs();
    end
end

function CleanUI_AdditionalItemButton_OnReceiveDrag(self)
    local infoType, info1, info2 = GetCursorInfo();

    if (infoType == "item" and info1 ~= nil) then
        updateButtonAttributes(self, info1)
    end

    ClearCursor();
end

function CleanUI_AdditionalItemButton_UpdateAlpha(self)
    local itemId = self:GetAttribute("itemId");

    local infoType, info1, info2 = GetCursorInfo();
    
    if (itemId ~= nil) then
        self:SetAlpha(1.0);
    elseif (infoType == "item" and info1 ~= nil) then
        self:SetAlpha(1.0);
    else
        self:SetAlpha(0.0);
    end
end
