local oldname = nil;
local equipmentSetItems = nil;

function CleanUI_InitGameMenu()
    CleanUI_GameMenuHookEvents();
    CleanUI_RegisterGameMenu();
    
    CleanUI_UpdateGameMenuBlackout();
    CleanUI_UpdateGameMenu();
end

function CleanUI_RegisterGameMenu()
    local frame = CreateFrame("Frame");

    --frame:RegisterEvent("...");

    frame:HookScript("OnEvent", CleanUI_GameMenuOnEvent); 
end

function CleanUI_GameMenuOnEvent(self, event, ...)
    --if (event == "..." or event == "...") then
    --    ...
    --end
end 

function CleanUI_GameMenuHookEvents()
    hooksecurefunc("GameMenuFrame_UpdateVisibleButtons", CleanUI_GameMenuFrame_UpdateVisibleButtons);

    hooksecurefunc(GameMenuFrame, "Show", CleanUI_GameMenuFrame_Show);
    hooksecurefunc(GameMenuFrame, "Hide", CleanUI_GameMenuFrame_Hide);
end

function CleanUI_GameMenuFrame_Show()
    UIFrameFadeOut(UIParent, 0.3, UIParent:GetAlpha(), 0.1);
    UIFrameFadeOut(ObjectiveTrackerFrame, 0.3, ObjectiveTrackerFrame:GetAlpha(), 0.1);
    UIFrameFadeOut(MinimapCluster, 0.3, MinimapCluster:GetAlpha(), 0.0);
    UIFrameFadeOut(Minimap, 0.3, Minimap:GetAlpha(), 0.1);
end

function CleanUI_GameMenuFrame_Hide()
    UIParent:SetAlpha(1);
    ObjectiveTrackerFrame:SetAlpha(1);
    MinimapCluster:SetAlpha(1);
    Minimap:SetAlpha(1);
end

function CleanUI_UpdateGameMenuBlackout()

    GameMenuFrame:SetParent(nil);
    GameMenuFrame:EnableMouse();
    
    if (not GameMenuFrame.blackout) then
        GameMenuFrame.blackout = CreateFrame("Frame", "GameMenuFrameBlackoutFrame", GameMenuFrame);
        GameMenuFrame.blackout:SetParent(GameMenuFrame);
        GameMenuFrame.blackout:SetFrameStrata("BACKGROUND");
        GameMenuFrame.blackout:EnableMouse();
        GameMenuFrame.blackout:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, 0);
        GameMenuFrame.blackout:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", 0, 0);

        GameMenuFrame.blackout:SetScript("OnMouseUp", function()  
            HideUIPanel(GameMenuFrame);
        end);

        GameMenuFrame.blackout:SetScript("OnShow", function()  
            CleanUI_UpdateGameMenuInfo();
        end);

        GameMenuFrame.blackout.bgInfo = GameMenuFrame.blackout:CreateFontString("CUI_GameMenuInfo", "ARTWORK");
        GameMenuFrame.blackout.bgInfo:SetSize(10, 10);
        GameMenuFrame.blackout.bgInfo:SetFontObject("GameFontHighlightLeft");
        GameMenuFrame.blackout.bgInfo:SetJustifyH("CENTER");
        GameMenuFrame.blackout.bgInfo:SetPoint("TOPLEFT", GameMenuFrame.blackout, "BOTTOMLEFT", 10, 30);
        GameMenuFrame.blackout.bgInfo:SetPoint("BOTTOMRIGHT", GameMenuFrame.blackout, "BOTTOMRIGHT", -10, 0);
    end
end

function CleanUI_UpdateGameMenuInfo()
    local infotext = "";

    local version, build, date, tocversion = GetBuildInfo();
    infotext = infotext..string.format("Version: |c00FFFF66%s|r, Build: |c00FFFF66%s|r, Date: |c00FFFF66%s|r, TOC-Version: |c00FFFF66%s|r                  ", version, build, date, tocversion);

    local memoryInUse = gcinfo()
    infotext = infotext.."Memory in use: |c00FFFF66" .. string.format("%.2f", memoryInUse) .. " kb|r                  ";

    local framerate = GetFramerate();
    infotext = infotext.."Your current FPS is |c00FFFF66" .. floor(framerate) .. "fps|r                  ";

    local down, up, lagHome, lagWorld = GetNetStats();
    infotext = infotext.."Connection Lag: |c00FFFF66" .. lagHome .. " ms|r, World Lag: |c00FFFF66" .. lagWorld .. " ms|r"; 

    GameMenuFrame.blackout.bgInfo:SetText(infotext);
    
    -- Currency Info

    local headersFound = 0;
    local numTokenTypes = C_CurrencyInfo.GetCurrencyListSize();

    local currencyInfoTable = {};

    for index=1, numTokenTypes do
        local currencyInfo = C_CurrencyInfo.GetCurrencyListInfo(index);
        
        if (currencyInfo.isHeader) then
            headersFound = headersFound + 1;
        end

        if (headersFound < 2) then
            tinsert(currencyInfoTable, currencyInfo);
        end
    end

    if (not GameMenuFrame.blackout.currenyInfo) then
        GameMenuFrame.blackout.currenyInfo = CreateFrame("Frame", "CUIGameMenuCurrencyInfo", GameMenuFrame.blackout);
        GameMenuFrame.blackout.currenyInfo:SetParent(GameMenuFrame.blackout);
        CleanUI_SetBackdrop(GameMenuFrame.blackout.currenyInfo);
    end

    local currencyFrameHeight = #currencyInfoTable * 20 + 50;

    GameMenuFrame.blackout.currenyInfo:ClearAllPoints();
    GameMenuFrame.blackout.currenyInfo:SetPoint("TOPLEFT", GameMenuFrame.blackout, "BOTTOMRIGHT", -315, currencyFrameHeight + 20);
    GameMenuFrame.blackout.currenyInfo:SetPoint("BOTTOMRIGHT", GameMenuFrame.blackout, "BOTTOMRIGHT", -10, 50);

    local offset = 0;
    for index=1, #currencyInfoTable do
        CleanUI_GameMenu_CreateCurrencyTag(currencyInfoTable[index], offset);
        offset = offset + 20;
    end
    
    -- Reputation

    headersFound = 0;
    local reputationTable = {};

    for factionIndex=1, NUM_FACTIONS_DISPLAYED, 1 do
        local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain = GetFactionInfo(factionIndex);
        
        if (isHeader) then
            headersFound = headersFound + 1;
        end

        if (headersFound < 3) then
            tinsert(reputationTable, factionIndex);
        end
    end

    if (not GameMenuFrame.blackout.reputationInfo) then
        GameMenuFrame.blackout.reputationInfo = CreateFrame("Frame", "CUIGameMenuReputationInfo", GameMenuFrame.blackout);
        GameMenuFrame.blackout.reputationInfo:SetParent(GameMenuFrame.blackout);
        CleanUI_SetBackdrop(GameMenuFrame.blackout.reputationInfo);
    end

    local reputationFrameHeight = #reputationTable * 20 + 20;

    GameMenuFrame.blackout.reputationInfo:ClearAllPoints();
    GameMenuFrame.blackout.reputationInfo:SetPoint("TOPLEFT", GameMenuFrame.blackout, "BOTTOMRIGHT", -315, currencyFrameHeight + 40 + reputationFrameHeight);
    GameMenuFrame.blackout.reputationInfo:SetPoint("BOTTOMRIGHT", GameMenuFrame.blackout, "BOTTOMRIGHT", -10, currencyFrameHeight + 40);

    local offset = 0;
    for index=1, #reputationTable do
        CleanUI_GameMenu_CreateReputationTag(reputationTable[index], offset);
        offset = offset + 20;
    end
end

function CleanUI_GameMenu_CreateCurrencyTag(currencyInfo, offset)

    local btnName = "CUI_GameMenuCurrency_"..(currencyInfo.iconFileID or "HEADER");

    local button = _G[btnName];
    if (not button) then
        button = CreateFrame("Frame", btnName, GameMenuFrame.blackout.currenyInfo, "CUIGameMenuCurrencyButtonTemplate");
        button:SetParent(GameMenuFrame.blackout.currenyInfo);
    end

    button:SetPoint("TOPLEFT", GameMenuFrame.blackout.currenyInfo, "TOPLEFT", 10, -10 - offset);

    if ( currencyInfo.isHeader ) then
        button.name:SetPoint("LEFT", 0, 0);
        button.count:SetText("");
        button.icon:SetTexture("");
        button.name:SetText(CURRENCY.." "..currencyInfo.name);
        button.name:SetFontObject("GameFontNormalSmall");
        button.name:SetTextHeight(12);
    else
        button.name:SetPoint("LEFT", 10, 0);
        button.count:SetText(BreakUpLargeNumbers(currencyInfo.quantity));
        button.count:SetFontObject("GameFontNormalSmall");
        button.icon:SetTexture(currencyInfo.iconFileID);
        button.icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
        
        --Gray out the text if the count is 0
        if ( currencyInfo.quantity == 0 ) then
            button.count:SetFontObject("GameFontDisableSmall");
            button.name:SetFontObject("GameFontDisableSmall");
        else
            button.count:SetFontObject("GameFontHighlightSmall");
            button.name:SetFontObject("GameFontHighlightSmall");
        end

        button.name:SetText(currencyInfo.name);
    end
end

function CleanUI_GameMenu_CreateReputationTag(index, offset)
    local btnName = "CUI_GameMenuReputation_"..index;

    local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain = GetFactionInfo(index);

    local button = _G[btnName];
    if (not button) then
        button = CreateFrame("Frame", btnName, GameMenuFrame.blackout.reputationInfo, "CUIGameMenuReputationButtonTemplate");
        button:SetParent(GameMenuFrame.blackout.reputationInfo);
        button.paragonReputation:SetScale(0.7);
    end

    button:SetPoint("TOPLEFT", GameMenuFrame.blackout.reputationInfo, "TOPLEFT", 10, -10 - offset);

    if ( isHeader ) then
        button.name:SetPoint("LEFT", 0, 0);
        button.name:SetText(name);
        button.name:SetFontObject("GameFontNormalSmall");

        button.standing:SetText("");

        button.paragonReputation:Hide();
    else
        button.name:SetPoint("LEFT", 10, 0);
        button.name:SetText(name);
        button.name:SetFontObject("GameFontHighlightSmall");
        button.standing:SetFontObject("GameFontNormalSmall");

        button.paragonReputation:Hide();

        local gender = UnitSex("player");
        local factionStandingText = GetText("FACTION_STANDING_LABEL"..standingID, gender);

        if ( factionID and C_Reputation.IsFactionParagon(factionID) ) then
            
            local currentValue, threshold, rewardQuestID, hasRewardPending = C_Reputation.GetFactionParagonInfo(factionID);

            local diff = threshold;
            local value = currentValue;
                
            while (value > diff) do
                value = value - diff;
            end

            local percent = 100 / diff * value;
            format("%.0f", percent)
            button.standing:SetText("Paragon ("..format("%.0f", percent).." %)");
            
            local color = FACTION_BAR_COLORS[standingID];
            button.standing:SetTextColor(0.3, 0.4, 1.0);

            if (hasRewardPending) then
                button.paragonReputation:Show();
            end

        elseif (standingID == 8) then

            button.standing:SetText("Paragon");
            button.standing:SetTextColor(0.6, 0.8, 1.0);

        else

            local friendID, friendRep, friendMaxRep, friendName, friendText, friendTexture, friendTextLevel, friendThreshold, nextFriendThreshold = GetFriendshipReputation(factionID);
            if (friendID ~= nil) then
                factionStandingText = friendTextLevel;
                barMin, barMax, barValue = friendThreshold, nextFriendThreshold, friendRep;
            end

            local percent = nil;

            if (barMax and barMin) then
                local diff = barMax - barMin;
                local value = barValue - barMin;
                percent = 100 / diff * value;
            end
            
            if (percent) then
                button.standing:SetText(factionStandingText.." ("..format("%.0f", percent).." %)");
            else
                button.standing:SetText(factionStandingText);
            end
            
            local color = FACTION_BAR_COLORS[standingID];
            button.standing:SetTextColor(color.r, color.g, color.b);

        end
    end
end

function CleanUI_UpdateGameMenu()
    CleanUI_RemoveTextures(GameMenuFrame);
    CleanUI_RemoveTextures(GameMenuFrame.Header);
    CleanUI_SetArtBackdrop(GameMenuFrame, 20, 0, 0, 0, true);

    CleanUI_GameMenuFrame_UpdateVisibleButtons();

    GameMenuFrame.Border:Hide();
    GameMenuFrame:SetScale(0.75);

    -- Change button style

    CleanUI_ChangeGameMenuButton(GameMenuButtonHelp);
    CleanUI_ChangeGameMenuButton(GameMenuButtonStore);
    CleanUI_ChangeGameMenuButton(GameMenuButtonWhatsNew);

    CleanUI_ChangeGameMenuButton(GameMenuButtonOptions);
    CleanUI_ChangeGameMenuButton(GameMenuButtonUIOptions);
    CleanUI_ChangeGameMenuButton(GameMenuButtonKeybindings);
    CleanUI_ChangeGameMenuButton(GameMenuButtonMacros);
    CleanUI_ChangeGameMenuButton(GameMenuButtonAddons);

    CleanUI_ChangeGameMenuButton(CleanUIGameMenuButtonReloadUI);

    CleanUI_ChangeGameMenuButton(GameMenuButtonLogout, true);
    CleanUI_ChangeGameMenuButton(GameMenuButtonQuit, true);

    CleanUI_ChangeGameMenuButton(GameMenuButtonContinue);
end

function CleanUI_ChangeGameMenuButton(button, warn)
    CleanUI_RemoveUiButtonTexture(button);

    button:SetNormalTexture("interface\\Buttons\\UI-Listbox-Highlight2");
    button:GetNormalTexture():SetTexCoord(0.0, 1.0, 0.0, 1.0);
    button:GetNormalTexture():SetAlpha(0.2);

    button:SetHighlightTexture("interface\\Buttons\\UI-Listbox-Highlight");
    button:GetHighlightTexture():SetTexCoord(0.0, 1.0, 0.0, 1.0);

    if (warn) then
        button:GetHighlightTexture():SetDesaturated(1);
        button:GetHighlightTexture():SetVertexColor(0.6, 0.0, 0.0);
    else
        button:GetHighlightTexture():SetDesaturated(1);
        button:GetHighlightTexture():SetVertexColor(0.0, 0.6, 0.2);
    end
end

function CleanUI_GameMenuFrame_UpdateVisibleButtons()
    local height = GameMenuFrame:GetHeight() + 55;
    GameMenuFrame:SetHeight(height);
    
    GameMenuButtonLogout:ClearAllPoints();
    GameMenuButtonLogout:SetPoint("TOP", CleanUIGameMenuButtonReloadUI, "BOTTOM", 0, -10);
    
    if (not GameMenuFrame.info) then
        local title = GetAddOnMetadata("CleanUI", "Title");
        local version = GetAddOnMetadata("CleanUI", "Version");
    
        GameMenuFrame.info = GameMenuFrame:CreateFontString("GameMenuFrameInfo", "ARTWORK");
        GameMenuFrame.info:SetFontObject("CuiFont_ExtraSmall");
        GameMenuFrame.info:SetText("powered by |c333399ff"..title.."|r ("..version..")");
        GameMenuFrame.info:SetJustifyH("CENTER");
        GameMenuFrame.info:SetSize(180, 15);
        GameMenuFrame.info:ClearAllPoints();
        GameMenuFrame.info:SetPoint("BOTTOMLEFT", GameMenuFrame, "BOTTOMLEFT", 0, 4);
        GameMenuFrame.info:SetPoint("BOTTOMRIGHT", GameMenuFrame, "BOTTOMRIGHT", 0, 4);
    end
    
    if (true) then
        return;
    end
    
    -- rearange buttons
    
    GameMenuFrame:SetSize(370, 345);
    
    GameMenuButtonHelp:SetSize(145, 27);  
    GameMenuButtonHelp:ClearAllPoints();
    GameMenuButtonHelp:SetPoint("TOPLEFT", GameMenuFrame, "TOPLEFT", 20, -30); 
    
    GameMenuButtonStore:SetSize(145, 27); 
    GameMenuButtonWhatsNew:SetSize(145, 27); 
    
    if (not GameMenuFrame.settingsLabel) then
        GameMenuFrame.settingsLabel = GameMenuFrame:CreateFontString("GameMenuFrameInfo", "ARTWORK");
        GameMenuFrame.settingsLabel:SetFontObject("GameFontNormal");
        GameMenuFrame.settingsLabel:SetText(SETTINGS);
        GameMenuFrame.settingsLabel:SetJustifyH("LEFT");
        GameMenuFrame.settingsLabel:SetSize(145, 27);
        GameMenuFrame.settingsLabel:ClearAllPoints();
        GameMenuFrame.settingsLabel:SetPoint("TOP", GameMenuButtonWhatsNew, "BOTTOM", 0, -16);
    end
    
    GameMenuButtonOptions:SetSize(145, 27); 
    GameMenuButtonOptions:SetPoint("TOP", GameMenuFrame.settingsLabel, "BOTTOM", 0, -1);
    GameMenuButtonUIOptions:SetSize(145, 27); 
    
    GameMenuButtonKeybindings:ClearAllPoints();
    GameMenuButtonKeybindings:SetPoint("TOP", GameMenuButtonUIOptions, "BOTTOM", 0, -16);
    GameMenuButtonKeybindings:SetSize(145, 27); 
    GameMenuButtonMacros:SetSize(145, 27); 
    GameMenuButtonAddons:SetSize(145, 27); 
    
    CleanUIGameMenuButtonReloadUI:SetSize(145, 27);  
    CleanUIGameMenuButtonReloadUI:ClearAllPoints();
    CleanUIGameMenuButtonReloadUI:SetPoint("TOPRIGHT", GameMenuFrame, "TOPRIGHT", -20, -170); 
    
    GameMenuButtonLogout:SetSize(145, 27); 
    GameMenuButtonLogout:ClearAllPoints();
    GameMenuButtonLogout:SetPoint("TOP", CleanUIGameMenuButtonReloadUI, "BOTTOM", 0, -16);
    GameMenuButtonQuit:SetSize(145, 27); 
    GameMenuButtonContinue:SetSize(145, 27); 
end
