local raidOpacity = 0.3 -- Set the visibility, 0 being invisible
local raid_frame = CreateFrame("Frame");

local groupsAndMembersArray = {}
local raidMembersArray = {}

function CleanUI_InitUnitFrames()
    CleanUIUserData.scaleUnitFrames = CleanUIUserData.scaleUnitFrames or 1.2
    CleanUIUserData.scalePartyFrames = CleanUIUserData.scalePartyFrames or 1.2
    CleanUIUserData.dragon = CleanUIUserData.dragon or 2

    CleanUI_UnitFramesHookEvents();
    CleanUI_RegisterUnitFramesEvents();
    
    CleanUI_ScaleUnitFrames();
    CleanUI_ScalePartyFrames();
    CleanUI_UpdateAlternateManaBar();
    CleanUI_UpdateUnitFrameDragon();

    CleanUI_UpdatePartyFrames();

    CleanUI_FixUnitFrameScales();
    
    --CleanUI_StartDelay(20, CleanUI_InitRaid);
    
    if (CompactRaidFrameManager) then
        CleanUI_UnitFramesHookRaidFrameManager();
        CleanUI_InitRaidFrameToggleButton();
    end
    
    -- aggro bar in player unit frame name area
    local aggroBar = CreateFrame("StatusBar", "PlayerFrameAggroBar", PlayerFrame, "CuiTextStatusBarTemplate");
    aggroBar:ClearAllPoints();
    aggroBar:SetPoint("TOPLEFT", PlayerFrameHealthBar, "TOPLEFT", 4, 18);
    aggroBar:SetPoint("BOTTOMRIGHT", PlayerFrameHealthBar, "TOPRIGHT", 0, -1);
    aggroBar:SetStatusBarColor(1.0, 0.0, 0.0);
    aggroBar:SetFrameStrata("LOW");
    
    PlayerFrameBackground:SetParent(aggroBar);
    PlayerFrameBackground:SetDrawLayer("BACKGROUND");
    PlayerFrameBackground:ClearAllPoints();
    PlayerFrameBackground:SetAllPoints(aggroBar);
    
    -- colors
    CleanUI_UpdatePlayerColors();
    CleanUI_UpdatePetColors();
    CleanUI_UpdatePartyFrameColors();

    --PetFrame:SetPoint("TOPLEFT", 60, -80);
    PetName:SetFontObject("GameFontNormalTiny");

    -- buff tooltip
    PartyMemberBuffTooltip:SetBackdrop(nil);
    CleanUI_SetBackdrop(PartyMemberBuffTooltip, true, nil, -3, 3, 3, -3);

    -- boss frames

    Boss2TargetFrame:SetPoint("TOPLEFT", Boss1TargetFrame, "BOTTOMLEFT", 0, 20);
    Boss3TargetFrame:SetPoint("TOPLEFT", Boss2TargetFrame, "BOTTOMLEFT", 0, 20);
    Boss4TargetFrame:SetPoint("TOPLEFT", Boss3TargetFrame, "BOTTOMLEFT", 0, 20);
    Boss5TargetFrame:SetPoint("TOPLEFT", Boss4TargetFrame, "BOTTOMLEFT", 0, 20);

    -- context menu

    table.remove(UnitPopupMenus["SELF"], #UnitPopupMenus["SELF"]); -- remove cancel
    DropDownList1Backdrop:SetAlpha(0);
    DropDownList1MenuBackdrop:SetAlpha(0);

    DropDownList2Backdrop:SetAlpha(0);
    DropDownList2MenuBackdrop:SetAlpha(0);

    CleanUI_SetBackdrop(DropDownList1);
    CleanUI_SetBackdrop(DropDownList2);

    -- replace unit frames

    CleanUI_InitUnitFramePositions();

    -- party info

    PlayerFrameGroupIndicatorLeft:Hide();
    PlayerFrameGroupIndicatorMiddle:Hide();
    PlayerFrameGroupIndicatorRight:Hide();

    CleanUI_InitDone();
end

function CleanUI_InitDone()
end

function CleanUI_FixUnitFrameScales()
    if (RuneFrame) then
        RuneFrame:SetFrameStrata("HIGH");
        RuneFrame:SetScale(0.9);
        RuneFrame:ClearAllPoints();
        RuneFrame:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 0, 38);
    end
end

function CleanUI_InitRaid()
    raid_frame:SetScript("OnUpdate", CleanUI_RaidOnUpdate);
    for i = 1,8 do
      groupsAndMembersArray[i] = {}
      for j = 1,5 do
         groupsAndMembersArray[i][j] = string.format("CompactRaidGroup%dMember%d", i,j)
      end
    end
    
    local raidMembersArray = {}
    for i = 1,40 do
      raidMembersArray[i] = string.format("CompactRaidFrame%d", i) 
    end
end

function CleanUI_RegisterUnitFramesEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    
    frame:RegisterEvent("RAID_ROSTER_UPDATE");
    frame:RegisterEvent("RAID_INSTANCE_WELCOME");
    
    frame:RegisterEvent("PLAYER_REGEN_DISABLED");
    frame:RegisterEvent("PLAYER_REGEN_ENABLED");
    
    frame:RegisterEvent("UNIT_PET");

    frame:HookScript("OnEvent", CleanUI_UnitFramesOnEvent);
end

function CleanUI_UnitFramesOnEvent(self, event, ...)

    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_CompactRaidFrames" ) then
            CleanUI_UnitFramesHookRaidFrameManager();
            CleanUI_InitRaidFrameToggleButton()
        end
    end

    if (event == "RAID_ROSTER_UPDATE") then
        CleanUI_ScalePartyFrames();
    end

    if (event == "PLAYER_REGEN_DISABLED") then
        CleanUI_UpdatePlayerColors();
        PlayerPortrait:SetVertexColor(1, 0, 0);
    end

    if (event == "PLAYER_REGEN_ENABLED") then
        CleanUI_UpdatePlayerColors();
        PlayerPortrait:SetVertexColor(1, 1, 1);
    end

    if (event == "UNIT_PET") then
        CleanUI_UpdatePetColors();
    end
    
    CleanUI_UpdatePartyFrameColors();
end

function CleanUI_UnitFramesHookEvents()
end

function CleanUI_UnitFramesHookRaidFrameManager()
    CleanUI_CompactRaidFrameManager_Collapse(CompactRaidFrameManager);

    hooksecurefunc("CompactRaidFrameManager_Expand", CleanUI_CompactRaidFrameManager_Expand);
    hooksecurefunc("CompactRaidFrameManager_Collapse", CleanUI_CompactRaidFrameManager_Collapse);
    
    hooksecurefunc("TargetFrame_CheckFaction", CleanUI_TargetFrame_CheckFaction);
    hooksecurefunc("UnitFrame_SetUnit", CleanUI_UpdatePartyFrameColors);
    hooksecurefunc("TargetofTarget_Update", CleanUI_UpdateTargetofTargetColors);
    
    hooksecurefunc("BossTargetFrame_OnLoad", CleanUI_BossTargetFrame_OnLoad);
    hooksecurefunc("CompactRaidFrameContainer_LayoutFrames", CleanUI_CompactRaidFrameContainer_LayoutFrames);

    hooksecurefunc("UnitPopup_ShowMenu", CleanUI_UnitFrames_AppendPositionSets);
    hooksecurefunc("CompactRaidFrameManager_Toggle", CleanUI_CompactRaidFrameManager_Toggle);

    hooksecurefunc("PetFrame_UpdateAnchoring", CleanUI_PetFrame_UpdateAnchoring); 

    hooksecurefunc("PartyMemberFrame_ToVehicleArt", CleanUI_PartyMemberFrame_ToVehicleArt); 
    hooksecurefunc("PartyMemberFrame_ToPlayerArt", CleanUI_PartyMemberFrame_ToPlayerArt);    

    local frame = CreateFrame("Frame");
    frame:HookScript("OnUpdate", CleanUI_UnitFrames_OnUpdate);

    CleanUI_PetFrame_UpdateAnchoring(PetFrame);
end

function CleanUI_PetFrame_UpdateAnchoring(self)
    if self.unit == "player" and PlayerVehicleHasComboPoints() then
	else
		local _, class = UnitClass("player");
		if ( class == "DEATHKNIGHT" or class == "ROGUE") then
		elseif ( class == "SHAMAN" or class == "DRUID" ) then
			self:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 90, -120);
		elseif ( class == "WARLOCK" ) then
			self:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 90, -90);
		elseif ( class == "PALADIN" ) then
			self:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 90, -90);
		elseif ( class == "PRIEST" ) then
			self:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 90, -90);
		elseif ( class == "MONK" ) then
			self:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 90, -100);
		end
	end
end

local UPDATE_WAIT = 0.05;
local wait = 0;
function CleanUI_UnitFrames_OnUpdate(self, elapsed)
    wait = wait + elapsed;

    if (wait > UPDATE_WAIT) then
        wait = 0;
        CleanUI_UnitFrames_DoUpdate(self);
        CleanUI_UpdateTargetofTargetColors(TargetFrameToT);
    end
end

function CleanUI_UnitFrames_DoUpdate(self, elapsed)
    CleanUI_UnitFrames_UpdateThreatSituation();
end

function CleanUI_UnitFrames_AppendPositionSets(dropdownMenu, which, unit, name, userData)

    if ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then
        local found = false;

        for index, value in ipairs(UnitPopupMenus[UIDROPDOWNMENU_MENU_VALUE]) do
            if( value == "UNLOCK_PLAYER_FRAME" or value == "UNLOCK_TARGET_FRAME" ) then
                found = true;
            end
        end

        if (not found) then
            return
        end

        UIDropDownMenu_AddSeparator(UIDROPDOWNMENU_MENU_LEVEL);

        local positions_header = UIDropDownMenu_CreateInfo();
        positions_header.text = UNITFRAMES_POSITIONS_PRESETS;
        positions_header.isNotRadio = true;
        positions_header.notCheckable = 1;
        positions_header.isTitle = true;

        UIDropDownMenu_AddButton(positions_header, UIDROPDOWNMENU_MENU_LEVEL);
        
        local battlePositions = UIDropDownMenu_CreateInfo();
        battlePositions.text = UNITFRAMES_BATTLE_POSITIONS;
        battlePositions.isNotRadio = true;
        battlePositions.notCheckable = 1;
        battlePositions.func = CleanUI_MoveFramesForBattle;

        UIDropDownMenu_AddButton(battlePositions, UIDROPDOWNMENU_MENU_LEVEL);
        
        local defaultPositions = UIDropDownMenu_CreateInfo();
        defaultPositions.text = UNITFRAMES_DEFAULT_POSITIONS;
        defaultPositions.isNotRadio = true;
        defaultPositions.notCheckable = 1;
        defaultPositions.func = CleanUI_MoveFramesDefault;

        UIDropDownMenu_AddButton(defaultPositions, UIDROPDOWNMENU_MENU_LEVEL);
    end
end

function CleanUI_InitUnitFramePositions()
    CleanUISavedVariables.UnitFramePositions = CleanUISavedVariables.UnitFramePositions or {};

    CleanUISavedVariables.UnitFramePositions.UNITFRAMES_POSITION_PRESET = CleanUISavedVariables.UnitFramePositions.UNITFRAMES_POSITION_PRESET or "DEFAULT";

    if (CleanUISavedVariables.UnitFramePositions.UNITFRAMES_POSITION_PRESET == "DEFAULT") then
        CleanUI_MoveFramesDefault();
    elseif (CleanUISavedVariables.UnitFramePositions.UNITFRAMES_POSITION_PRESET == "BATTLE") then
        CleanUI_MoveFramesForBattle();
    end
end

function CleanUI_MoveFramesForBattle()
	PlayerFrame:ClearAllPoints();
    PlayerFrame:SetUserPlaced(true);
    PlayerFrame:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOM", -150, 200);
	TargetFrame:ClearAllPoints();
    TargetFrame:SetUserPlaced(true);
    TargetFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOM", 150, 200);

    -- CompactRaidFrameManager:SetMovable(true);
    -- CompactRaidFrameManager:SetUserPlaced(true);
    CompactRaidFrameManager:ClearAllPoints();
    CompactRaidFrameManager:SetPoint("TOPLEFT", UIParent, "TOPLEFT", -182, -12);

    CleanUISavedVariables.UnitFramePositions.UNITFRAMES_POSITION_PRESET = "BATTLE";
    CleanUISavedVariables.UnitFramePositions.RAID_OFFSET = -12;
end


function CleanUI_MoveFramesDefault()
    PlayerFrame_ResetUserPlacedPosition();
    TargetFrame_ResetUserPlacedPosition();

    -- CompactRaidFrameManager:SetMovable(true);
    -- CompactRaidFrameManager:SetUserPlaced(false);
    -- CompactRaidFrameManager:SetMovable(false);

    CompactRaidFrameManager_Collapse(CompactRaidFrameManager);

    CleanUISavedVariables.UnitFramePositions.UNITFRAMES_POSITION_PRESET = "DEFAULT";
    CleanUISavedVariables.UnitFramePositions.RAID_OFFSET = nil;
end

function CleanUI_CompactRaidFrameManager_Toggle(self)
    local offset = CleanUISavedVariables.UnitFramePositions.RAID_OFFSET
    if (offset) then
        if ( self.collapsed ) then
            self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", -182, offset);
        else
            self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", -7, offset);
        end
    end
end

local UPDATE_WAIT = 0.05;
local wait = 0;
function CleanUI_RaidOnUpdate(self, elapsed)
    wait = wait + elapsed;

    if (wait > UPDATE_WAIT) then
        wait = 0;
        CleanUI_RaidDoUpdate(self);
    end
end

local function set_opacity(raidmember)
    if raidmember then
        local transparency;
    
        if raidmember:GetAlpha() < 0.8 then
            transparency = raidOpacity;
        else
            transparency = 1;
        end
        
        select(1,raidmember:GetRegions()):SetAlpha(transparency); -- Background
        select(7,raidmember:GetRegions()):SetAlpha(transparency); -- Name Text
        select(1,raidmember:GetChildren()):SetAlpha(transparency); -- Health Bar
        select(2,raidmember:GetChildren()):SetAlpha(transparency); -- Mana Bar
    end
end

function CleanUI_RaidDoUpdate(self)
    --Keep Groups Together Unchecked
    for i=1, 40 do
        local raidmember = _G[raidMembersArray[i]];
        set_opacity(raidmember);
    end
    
      -- Keep Groups Together Checked
    for i=1, 8 do
        for j=1,5 do
            local raidmember = _G[groupsAndMembersArray[i][j]];
            set_opacity(raidmember);
        end
    end
end

function CleanUI_InitRaidFrameToggleButton()
    local button = _G["CUIRaidFrameToggleButton"];
    if (not button) then
        button = CreateFrame("Button", "CUIRaidFrameToggleButton", CompactRaidFrameManager, "UIPanelButtonTemplate");
        CleanUI_RemoveUiButtonTexture(button);
        button:SetSize(12, 12);
        button:SetNormalTexture("Interface\\RaidFrame\\RaidPanel-Toggle");
        button:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
        button:SetPoint("BOTTOMRIGHT", CompactRaidFrameManager, "TOPRIGHT", -6, -3);

        button:SetScript("OnClick", CleanUI_ToggleRaidFrame);
    end

    CleanUI_ToggleRaidFrame();
    CleanUI_UpdateRaidFrameToggleButtonText();
end

local raidFramesHidden = true;

function CleanUI_ToggleRaidFrame()

    CompactRaidFrameContainer:ClearAllPoints();

    if (raidFramesHidden) then
        CompactRaidFrameContainer:SetPoint("TOPLEFT", CompactRaidFrameManager, "TOPRIGHT", 5, -10);
    else
        CompactRaidFrameContainer:SetPoint("TOPLEFT", CompactRaidFrameManager, "TOPRIGHT", -10000, 0);
    end
    
    raidFramesHidden = not raidFramesHidden;

    CleanUI_UpdateRaidFrameToggleButtonText();
end

function CleanUI_UpdateRaidFrameToggleButtonText()
    local button = _G["CUIRaidFrameToggleButton"];

    if (button) then
        if (raidFramesHidden) then
        --button:SetText("-");
        button:GetNormalTexture():SetTexCoord(0.25, 0.5, 0.4, 0.6);
        else
        --button:SetText("+");
        button:GetNormalTexture():SetTexCoord(0.75, 1, 0.4, 0.6);
        end

        if (IsInRaid()) then
            button:Show();
        else
            button:Hide();
        end
    end
end

function CleanUI_CompactRaidFrameContainer_LayoutFrames(self)
    CleanUI_UpdateRaidFrameToggleButtonText();
end

function CleanUI_UpdatePlayerColors()
    PlayerName:SetTextColor(1.0, 1.0, 1.0, 1.0);
    
    if (InCombatLockdown()) then
        PlayerName:SetTextColor(1.0, 0.0, 0.0, 1.0);
    end
end

function CleanUI_BossTargetFrame_OnLoad(self, unit, event)
    local classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
    
    if (unit) then
        local localizedClass, englishClass = UnitClass(unit);
        classColor = RAID_CLASS_COLORS[englishClass];
    
        if (not classColor) then
            classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
        end
    end
    
    self.borderTexture:SetDesaturated(1);
    self.borderTexture:SetVertexColor(classColor.r, classColor.g, classColor.b);
end

function CleanUI_UpdatePartyFrameColors()
    local partyFrame, partyFrameName, partyFrameTexture;
    for i=1, 5 do
        partyFrame = _G["PartyMemberFrame"..i];
        partyFrameName = _G["PartyMemberFrame"..i.."Name"];
        partyFrameTexture = _G["PartyMemberFrame"..i.."Texture"];
        
        if (partyFrame) then
        
            local classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
            local unit = partyFrame.unit;
            
            if (unit) then
                local localizedClass, englishClass = UnitClass(unit);
                classColor = RAID_CLASS_COLORS[englishClass];
            
                if (not classColor) then
                    classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
                end
            end
            
            partyFrameName:SetTextColor(classColor.r, classColor.g, classColor.b, 1.0);
            
            partyFrameTexture:SetDesaturated(1);
            partyFrameTexture:SetVertexColor(classColor.r, classColor.g, classColor.b);
        end
    end
end

function CleanUI_UpdatePetColors()
    local classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
    local unit = "pet";
    
    if (unit) then
        local localizedClass, englishClass = UnitClass(unit);
        classColor = RAID_CLASS_COLORS[englishClass];
    
        if (not classColor) then
            classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
        end
    end
            
    PetFrameTexture:SetDesaturated(1);
    PetFrameTexture:SetVertexColor(classColor.r, classColor.g, classColor.b);
end

function CleanUI_UpdateTargetofTargetColors(self, elapsed)
    if (not UnitExists(self.unit)) then
        return;
    end
    
    local texture = _G[self:GetName().."TextureFrameTexture"];
    
    local localizedClass, englishClass = UnitClass(self.unit);
    local classColor = RAID_CLASS_COLORS[englishClass];

    if (not classColor) then
        classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
    end
            
    texture:SetDesaturated(1);
    texture:SetVertexColor(classColor.r, classColor.g, classColor.b);
end

function CleanUI_UpdateAlternateManaBar()
    if (not PlayerFrameAlternateManaBar) then
        return;
    end

    PlayerFrameAlternateManaBar.MonkBackground:Hide();
    PlayerFrameAlternateManaBar.DefaultBorder:Hide();
    PlayerFrameAlternateManaBar.DefaultBorderLeft:Hide();
    PlayerFrameAlternateManaBar.DefaultBorderRight:Hide();
    PlayerFrameAlternateManaBar.MonkBorder:Hide();
    
    if (not PlayerFrameAlternateManaBar.cuiborder) then
        PlayerFrameAlternateManaBar.cuiborder = PlayerFrameAlternateManaBar:CreateTexture("OVERLAY");
        PlayerFrameAlternateManaBar.cuiborder:SetPoint("TOPLEFT", PlayerFrameAlternateManaBar, "TOPLEFT", -2, 2);
        PlayerFrameAlternateManaBar.cuiborder:SetPoint("BOTTOMRIGHT", PlayerFrameAlternateManaBar, "BOTTOMRIGHT", 2, -2);
        PlayerFrameAlternateManaBar.cuiborder:SetTexture("Interface\\Tooltips\\UI-StatusBar-Border");
        PlayerFrameAlternateManaBar.cuiborder:SetDrawLayer("OVERLAY", 7);
    end
    
    PlayerFrameAlternateManaBar:ClearAllPoints();
    
    PlayerFrameAlternateManaBar:SetPoint("TOPLEFT", PlayerFrameManaBar, "BOTTOMLEFT", 15, -5);
    PlayerFrameAlternateManaBar:SetPoint("BOTTOMRIGHT", PlayerFrameManaBar, "BOTTOMRIGHT", -15, -13);
end

function CleanUI_UnitFrames_UpdateThreatSituation()
    -- player

    local isTanking, status, threatpct, rawthreatpct, threatvalue = UnitDetailedThreatSituation("player", "playertarget");
    
    if (not threatpct) then
        threatpct = 0;
    end

    if (threatpct > 100) then
        threatpct = 100;
    end

    PlayerFrameAggroBar:SetValue(threatpct);

    -- party
    
    local frame, unit, target, healthBar, manaBar, debuff1, nameText;

    for i=1, 5 do
        frame = _G["PartyMemberFrame"..i];
        unit = "party"..i;

        if (UnitExists(unit) and frame and frame.aggroBar) then
            target = "party"..i.."target";

            if (IsAltKeyDown()) then
                target = "playertarget";
            end    
        
            isTanking, status, threatpct, rawthreatpct, threatvalue = UnitDetailedThreatSituation(unit, target);
        
            if (not threatpct) then
                threatpct = 0;
            end

            if (threatpct > 100) then
                threatpct = 100;
            end

            frame.aggroBar:SetValue(threatpct);
        end
    end
end

function CleanUI_CompactRaidFrameManager_Expand(self)
    self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", -7, -175);
end

function CleanUI_CompactRaidFrameManager_Collapse(self)
    self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", -182, -175);
end

function CleanUI_TargetFrame_CheckFaction(self)
    local name = _G[self:GetName().."TextureFrameName"];
    
    local unitName = GetUnitName(self.unit);
    
    -- change alpha, when tapped
    if ( not UnitPlayerControlled(self.unit) and UnitIsTapDenied(self.unit) ) then
        name:SetTextColor(1.0, 1.0, 1.0, 1.0);
        TargetFrameHealthBar:SetAlpha(0.3);
        TargetFramePortrait:SetAlpha(0.5);
        TargetFramePortrait:SetDesaturated(1);
    else
        TargetFrameHealthBar:SetAlpha(1.0);
        TargetFramePortrait:SetAlpha(1.0);
        TargetFramePortrait:SetDesaturated(nil);
    end
    
    -- background color == class color
    self.nameBackground:SetVertexColor(0.5, 0.5, 0.5);
    
    local localizedClass, englishClass = UnitClass(self.unit);
    local classColor = RAID_CLASS_COLORS[englishClass];

    if (not classColor) then
        classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
    end
        
    self.nameBackground:SetVertexColor(classColor.r, classColor.g, classColor.b);
    
    self.textureFrame.texture:SetDesaturated(1);
    self.textureFrame.texture:SetVertexColor(classColor.r, classColor.g, classColor.b);
    
    -- change text color
    if (UnitPlayerControlled(self.unit) and not UnitIsPVP(self.unit)) then
        name:SetTextColor(1.0, 1.0, 1.0, 1.0);
    elseif (UnitPlayerControlled(self.unit) and UnitIsPVP(self.unit) and UnitFactionGroup("player") == UnitFactionGroup(self.unit)) then
        -- same faction
        name:SetTextColor(1.0, 1.0, 1.0, 1.0);
    elseif (UnitPlayerControlled(self.unit) and UnitIsPVP(self.unit) and UnitFactionGroup("player") ~= UnitFactionGroup(self.unit)) then
        -- other faction
        local englishFaction, localizedFaction = UnitFactionGroup(self.unit);

        if (englishFaction == "Horde") then
            unitName = "|TInterface/WorldStateFrame/HordeIcon:20|t"..unitName;
        elseif (englishFaction == "Alliance") then
            unitName = "|TInterface/WorldStateFrame/AllianceIcon:20|t"..unitName;
        end
        
        local red, green, blue, alpha = UnitSelectionColor(self.unit);
        name:SetTextColor(red, green, blue, alpha);
    else
        local red, green, blue, alpha = UnitSelectionColor(self.unit);
        name:SetTextColor(red, green, blue, alpha);
    end
        
    name:SetText(unitName);
end

function CleanUI_SetUnitFrameDragon(id)
    CleanUIUserData.dragon = id;
    CleanUI_UpdateUnitFrameDragon();
end

function CleanUI_SetUnitFrameColoredDragon(coloredDragon)
    CleanUIUserData.coloredDragon = coloredDragon;
end

function CleanUI_UpdateUnitFrameDragon()

    PlayerFrameTexture:SetDesaturated(nil);
    PlayerFrameTexture:SetVertexColor(1.0, 1.0, 1.0);
        
    if (CleanUIUserData.dragon == 1) then
        PlayerFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Elite");
    elseif (CleanUIUserData.dragon == 2) then
        PlayerFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Rare");
    elseif (CleanUIUserData.dragon == 3) then
        PlayerFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Rare-Elite");
    else
        PlayerFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame");
    end

    local localizedClass, englishClass = UnitClass("player");
    local classColor = RAID_CLASS_COLORS[englishClass];

    if (not classColor) then
        classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
    end

    local desaturated = nil;
    local vertexColor = {r = 1, g = 1, b = 1};

    if (CleanUIUserData.coloredDragon) then
        desaturated = 1;
        vertexColor = classColor;
    end
    
    PlayerFrameTexture:SetDesaturated(desaturated);
    PlayerFrameTexture:SetVertexColor(vertexColor.r, vertexColor.g, vertexColor.b);
    
    if (PlayerFrameAlternateManaBar.cuiborder) then
        PlayerFrameAlternateManaBar.cuiborder:SetDesaturated(desaturated);
        PlayerFrameAlternateManaBar.cuiborder:SetVertexColor(vertexColor.r, vertexColor.g, vertexColor.b);
    end
    
    if (ComboPointPlayerFrame) then
        ComboPointPlayerFrame.Background:SetDesaturated(desaturated);
        ComboPointPlayerFrame.Background:SetVertexColor(vertexColor.r, vertexColor.g, vertexColor.b);
    end
    
    if (PaladinPowerBarFrame) then
        PaladinPowerBarFrameBankBG:SetDesaturated(desaturated);
        PaladinPowerBarFrameBankBG:SetVertexColor(vertexColor.r, vertexColor.g, vertexColor.b);
        PaladinPowerBarFrameBG:SetDesaturated(desaturated);
        PaladinPowerBarFrameBG:SetVertexColor(vertexColor.r, vertexColor.g, vertexColor.b);
    end
    
    if (WarlockPowerFrame) then
        CleanUI_ReplaceAllTextures("Interface\\PlayerFrame\\UI-WarlockShard", WarlockPowerFrame, "Interface\\AddOns\\CleanUI\\skins\\frame\\CUI-WarlockShard");
    end
end

function CleanUI_SetUnitFramesScale(scale)
    if (scale and scale >= 0.75 and scale <= 2) then
        CleanUIUserData.scaleUnitFrames = scale;
        CleanUI_ScaleUnitFrames() 
    else
        DEFAULT_CHAT_FRAME:AddMessage("scale: from 0.75 to 2.0", 1.0, 0, 0, 1, 10);
    end
end

function CleanUI_ScaleUnitFrames()    
    PlayerFrame:SetScale(CleanUIUserData.scaleUnitFrames);
    TargetFrame:SetScale(CleanUIUserData.scaleUnitFrames);
    FocusFrame:SetScale(CleanUIUserData.scaleUnitFrames);
end

function CleanUI_SetPartyScale(scale)
    if (scale and scale >= 0.75 and scale <= 2) then
        CleanUIUserData.scalePartyFrames = scale;
        CleanUI_ScalePartyFrames();
    else
        DEFAULT_CHAT_FRAME:AddMessage("scale: from 0.75 to 2.0", 1.0, 0, 0, 1, 10);
    end
end

function CleanUI_ScalePartyFrames()
    local frame;
    for i=1, 5 do
        frame = _G["PartyMemberFrame"..i];
        
        if (frame) then
            frame:SetScale(CleanUIUserData.scalePartyFrames);
        end
    end
end

function CleanUI_PartyMemberFrame_ToPlayerArt(self)
    local parentName = self:GetName();

    _G[parentName.."Texture"]:SetTexture("Interface\\TargetingFrame\\UI-FocusTargetingFrame")
    _G[parentName.."Flash"]:SetTexture("Interface\\AddOns\\CleanUI\\skins\\frame\\partyframe-flash")
    _G[parentName.."Flash"]:SetPoint("TOPLEFT", -2, 5);

    _G[parentName.."HealthBarText"]:SetPoint("CENTER", 20, 2);
    _G[parentName.."HealthBarTextLeft"]:SetPoint("LEFT", 42, 2);
    _G[parentName.."HealthBarTextRight"]:SetPoint("RIGHT", -10, 2);

    _G[parentName.."ManaBarText"]:SetPoint("CENTER", 20, -8);
    _G[parentName.."ManaBarTextLeft"]:SetPoint("LEFT", 42, -8);
    _G[parentName.."ManaBarTextRight"]:SetPoint("RIGHT", -10, -8);

    background = _G[parentName.."Background"];
    background:ClearAllPoints();
    background:SetPoint("TOPLEFT", 47, -7);
    background:SetPoint("BOTTOMRIGHT", self, "TOPLEFT", 116, -40);
    
    _G[parentName.."HealthBar"]:SetPoint("TOPLEFT", 47, -22);
    _G[parentName.."ManaBar"]:SetPoint("TOPLEFT", 47, -32);
    _G[parentName.."Debuff1"]:SetPoint("TOPLEFT", 48, -44);

    nameText = _G[parentName.."Name"];

    nameText:ClearAllPoints();
    nameText:SetPoint("BOTTOMLEFT", 50, 33);
    nameText:SetPoint("TOPRIGHT", self, "BOTTOMLEFT", 110, 45);
    nameText:SetJustifyH("CENTER");
    nameText:SetFontObject("GameFontNormalTiny");

    if (self.aggroBar) then
        self.aggroBar:Show();
    end
end

function CleanUI_PartyMemberFrame_ToVehicleArt(self, vehicleType)
    local parentName = self:GetName();

    _G[parentName.."Texture"]:SetTexture("Interface\\TargetingFrame\\UI-FocusTargetingFrame")
    _G[parentName.."Flash"]:SetTexture("Interface\\AddOns\\CleanUI\\skins\\frame\\partyframe-flash")
    _G[parentName.."Flash"]:SetPoint("TOPLEFT", -2, 5);

    _G[parentName.."HealthBarText"]:SetPoint("CENTER", 20, 12);
    _G[parentName.."HealthBarTextLeft"]:SetPoint("LEFT", 42, 12);
    _G[parentName.."HealthBarTextRight"]:SetPoint("RIGHT", -10, 12);

    _G[parentName.."ManaBarText"]:SetPoint("CENTER", 20, 2);
    _G[parentName.."ManaBarTextLeft"]:SetPoint("LEFT", 42, 2);
    _G[parentName.."ManaBarTextRight"]:SetPoint("RIGHT", -10, 2);

    background = _G[parentName.."Background"];
    background:ClearAllPoints();
    background:SetPoint("TOPLEFT", 45, -11);
    
    _G[parentName.."HealthBar"]:SetPoint("TOPLEFT", 47, -12);
    _G[parentName.."ManaBar"]:SetPoint("TOPLEFT", 47, -21);
    _G[parentName.."Debuff1"]:SetPoint("TOPLEFT", 48, -32);

    nameText = _G[parentName.."Name"];

    nameText:ClearAllPoints();
    nameText:SetPoint("BOTTOMLEFT", 47, 40);
    nameText:SetPoint("TOPRIGHT", self, "BOTTOMLEFT", 110, 55);
    nameText:SetJustifyH("CENTER");
    nameText:SetFontObject("GameFontNormalTiny");

    if (self.aggroBar) then
        self.aggroBar:Hide();
    end
end

function CleanUI_UpdatePartyFrames()
    local frame, background, healthBar, manaBar, debuff1, nameText;
    for i=1, 5 do
        frame = _G["PartyMemberFrame"..i];
        
        if (frame) then
            CleanUI_PartyMemberFrame_ToPlayerArt(frame);


            -- aggro bar in player unit frame name area
            frame.aggroBar = CreateFrame("StatusBar", "PartyMemberFrame"..i.."AggroBar", frame, "CuiTextStatusBarTemplate");
            frame.aggroBar:ClearAllPoints();
            frame.aggroBar:SetPoint("TOPLEFT", frame, "TOPLEFT", 47, -9);
            frame.aggroBar:SetPoint("BOTTOMRIGHT", frame, "TOPLEFT", 116, -20);
            frame.aggroBar:SetStatusBarColor(1.0, 0.0, 0.0);
            frame.aggroBar:SetFrameStrata("LOW");
        end
    end
end